#ifndef __CWAVEFUNCTION_WF_PP_PHASESHIFT_CC__
#define __CWAVEFUNCTION_WF_PP_PHASESHIFT_CC__
#include "wavefunction.h"


CWaveFunction_pp_phaseshift::CWaveFunction_pp_phaseshift(string  parsfilename) : CWaveFunction() {
  int iq,ichannel;
  double q;
  ParsInit(parsfilename);

  m1=MPROTON;
  m2=m1;
	IDENTICAL=1;
  q1q2=1;
  if(COULOMB==0) q1q2=0;
  nchannels=4;

  ellmax=1;
  InitArrays();
  printf("Arrays Initialized\n");
  ell[0]=0;
  ell[1]=ell[2]=ell[3]=1;

  InitWaves();
  printf("Partial Waves Initialized\n");

  // Channel weight is (2J+1)/[(2s1+1)*(2s2+1)]
  channelweight[0]=2*0.25;
  channelweight[1]=2*0.25;
  channelweight[2]=2*0.75;
  channelweight[3]=2*1.25;
  read_phaseshifts();
  printf("phaseshifts read in\n");

  for(ichannel=0;ichannel<nchannels;ichannel++){
    for(iq=0;iq<nqmax;iq++){
      q=qarray[iq];
      /*if(ichannel==0) printf("q=%g, ddeltadq=%g, W=%g, W0=%g\n",
	qarray[iq],ddeltadq[ichannel][iq],
	GetIW(ell[ichannel],epsilon,q,q1q2,eta[iq],delta[ichannel][iq]),
	GetIW(ell[ichannel],epsilon,q,q1q2,eta[iq],0.0));*/
      Wepsilon[ichannel][iq]=ddeltadq[ichannel][iq]
	-GetIW(ell[ichannel],epsilon,q,q1q2,eta[iq],delta[ichannel][iq])
	+GetIW(ell[ichannel],epsilon,q,q1q2,eta[iq],0.0);
      Wepsilon[ichannel][iq]=3.0*Wepsilon[ichannel][iq]
	*pow(HBARC/epsilon,3)/(2.0*q*q);
      /*if(ichannel==0){
	printf("q=%g, W=%g, wprime=%g\n",q,Wepsilon[ichannel][iq],
	ddeltadq[ichannel][iq]*3.0*pow(HBARC/epsilon,3)/(2.0*q*q));
	}
	else{
	Wepsilon[ichannel][iq]=0.0;
	delta[ichannel][iq]=0.0;
	}*/
    }
  }
  printf("Initialization finished\n");
}

double CWaveFunction_pp_phaseshift::CalcPsiSquared(int iq,double r,double ctheta){
  double psisquared,x,dpsi2,q,theta;
  double delta_1s0,delta_3p0,delta_3p1,delta_3p2;
  complex<double> psi,psisymm,psianti,psia,psib,hstar0,hstar1;
  complex<double> Xlm00,Xlm10,Xlm11;
  int ichannel;

  q=qarray[iq];
  if(iq>=nqmax){
    printf("iq too large!\n");
    exit(1);
  }
  psia=planewave[iq]->planewave(r,ctheta);
  psib=planewave[iq]->planewave(r,-ctheta);
  psisymm=(1.0/sqrt(2.0))*(psia+psib);
  psianti=(1.0/sqrt(2.0))*(psia-psib);
  
  if(STRONG==1){
    if(r<epsilon){
      psisquared=0.25*real(psisymm*conj(psisymm))
	+0.75*real(psianti*conj(psianti));
      for(ichannel=0;ichannel<nchannels;ichannel++){
	dpsi2=channelweight[ichannel]*Wepsilon[ichannel][iq];
	psisquared+=dpsi2;
      }
    }
    else{
      theta=acos(ctheta);
      x=q*r/HBARC;
      // Notation is (2S+1)-L-J
      delta_1s0=delta[0][iq];
      delta_3p0=delta[1][iq];
      delta_3p1=delta[2][iq];
      delta_3p2=delta[3][iq];
      hstar0=partwave[0][iq]->GetPhiIncoming(r)/x;
      hstar1=partwave[1][iq]->GetPhiIncoming(r)/x;
      // XlmLM 9s Y_{LM}*(1/2)*i^L*sqrt(4*PI*(2*L+1))*hstar_L
      Xlm00=0.5*sqrt(2.0)*sqrt(4.0*PI)*SpherHarmonics::Ylm(0,0,theta,0.0)*hstar0;
      Xlm10=ci*0.5*sqrt(2.0)*sqrt(12.0*PI)*SpherHarmonics::Ylm(1,0,theta,0.0)*hstar1;
      Xlm11=ci*0.5*sqrt(2.0)*sqrt(12.0*PI)*SpherHarmonics::Ylm(1,1,theta,0.0)*hstar1;
      
      // First do the case for S=0;
      psi=psisymm;
      // this refers to S=0, L=0, J=0 channel
      psi+=Xlm00*(Misc::ceiphi(-2.0*delta_1s0)-1.0);
      // S=0, L=1, J=1
      psisquared=0.25*real(psi*conj(psi));
      
      // Now let's do the case for S=1, M_S=+1
      psi=psianti;
      // S=1, L=1 and J=1,2,
      psi+=Xlm10*(0.5*Misc::ceiphi(-2.0*delta_3p1)
		  +0.5*Misc::ceiphi(-2.0*delta_3p2)-1.0);
      psia=Xlm11*0.5*(Misc::ceiphi(-2.0*delta_3p1)-Misc::ceiphi(-2.0*delta_3p2));
      psisquared+=0.5*real(psi*conj(psi)+psia*conj(psia));
      // Term is doubled to account for M_S=-1
      
      // Now let's do the case with S=1, M_S=0;
      psi=psianti;
      // S=1, L=1, J=0,2
      psi+=Xlm10*((2.0/3.0)*Misc::ceiphi(-2.0*delta_3p2)
		  +(1.0/3.0)*Misc::ceiphi(-2.0*delta_3p0)-1.0);
      psia=Xlm11*(Misc::ceiphi(-2.0*delta_3p2)-Misc::ceiphi(-2.0*delta_3p0))/3.0;
      psisquared+=0.25*real(psi*conj(psi)+2.0*psia*conj(psia));
      
    }
  }
  else psisquared=0.25*real(psisymm*conj(psisymm))
	 +0.75*real(psianti*conj(psianti));
  if(r>epsilon && psisquared<-0.001){
    printf("in CalcPsiSquared psisquared=%g,q=%g,r=%g\n",psisquared,q,r);
    if(r>epsilon) exit(1);
  }
  //if(psisquared<0.0){
  //  printf("psisquared <0, = %g, r=%g, q=%g\n",psisquared,r,q);
  //}
	psisquared*=RelativisticCorrection(r,iq);
  return psisquared;

}

void CWaveFunction_pp_phaseshift::read_phaseshifts(){
//#include "pp_phaseshiftdat.cc"
   double delqdata=0.5;
   int nqdata=400;
   double data_delta[4][401]={{0,2.35616e-11,3.86529e-10,5.01672e-07,1.81454e-05,0.000156631,0.000661218,0.00185686,0.00404566,0.00744801,0.0121928,0.0183324,0.0258643,0.0347494,0.0449264,0.0563214,0.0688534,0.0824388,0.0969931,0.112432,0.128673,0.145635,0.163236,0.1814,0.200048,0.219108,0.238505,0.258169,0.278033,0.29803,0.318098,0.338177,0.358212,0.378149,0.397939,0.417538,0.436903,0.455998,0.474789,0.493246,0.511343,0.529059,0.546373,0.563271,0.57974,0.595771,0.611357,0.626493,0.641178,0.65541,0.669192,0.682526,0.695417,0.707871,0.719895,0.731495,0.74268,0.75346,0.763843,0.77384,0.783459,0.792712,0.801608,0.810158,0.818372,0.826261,0.833834,0.841101,0.848074,0.85476,0.86117,0.867312,0.873197,0.878832,0.884227,0.889389,0.894326,0.899047,0.903559,0.907869,0.911984,0.915911,0.919656,0.923227,0.926628,0.929867,0.932948,0.935877,0.938659,0.9413,0.943804,0.946175,0.94842,0.950541,0.952542,0.954429,0.956205,0.957873,0.959437,0.960901,0.962268,0.963541,0.964723,0.965817,0.966827,0.967754,0.968601,0.969371,0.970067,0.97069,0.971243,0.971729,0.972148,0.972504,0.972799,0.973033,0.97321,0.97333,0.973396,0.973408,0.97337,0.973281,0.973145,0.972961,0.972732,0.972458,0.972142,0.971784,0.971385,0.970947,0.97047,0.969957,0.969407,0.968823,0.968204,0.967551,0.966867,0.966151,0.965405,0.964628,0.963823,0.96299,0.962129,0.961242,0.960328,0.959389,0.958426,0.957439,0.956428,0.955395,0.954339,0.953261,0.952163,0.951044,0.949905,0.948747,0.947569,0.946373,0.945159,0.943927,0.942678,0.941413,0.94013,0.938832,0.937519,0.93619,0.934846,0.933487,0.932115,0.930729,0.929329,0.927916,0.92649,0.925052,0.923601,0.922139,0.920664,0.919179,0.917682,0.916174,0.914655,0.913126,0.911587,0.910038,0.908479,0.906911,0.905333,0.903747,0.902151,0.900547,0.898934,0.897313,0.895684,0.894047,0.892402,0.890749,0.889089,0.887422,0.885748,0.884066,0.882378,0.880684,0.878982,0.877275,0.875561,0.873841,0.872115,0.870383,0.868646,0.866903,0.865154,0.8634,0.861641,0.859877,0.858108,0.856334,0.854555,0.852772,0.850983,0.849191,0.847394,0.845592,0.843787,0.841977,0.840163,0.838346,0.836524,0.834699,0.83287,0.831038,0.829201,0.827362,0.825519,0.823673,0.821823,0.819971,0.818115,0.816256,0.814395,0.81253,0.810663,0.808793,0.80692,0.805044,0.803166,0.801286,0.799403,0.797517,0.79563,0.793739,0.791847,0.789953,0.788056,0.786157,0.784257,0.782354,0.780449,0.778542,0.776634,0.774724,0.772812,0.770898,0.768983,0.767066,0.765147,0.763227,0.761305,0.759382,0.757458,0.755532,0.753604,0.751676,0.749746,0.747815,0.745882,0.743949,0.742014,0.740078,0.738141,0.736203,0.734264,0.732324,0.730384,0.728442,0.726499,0.724556,0.722611,0.720666,0.71872,0.716773,0.714826,0.712877,0.710929,0.708979,0.707029,0.705078,0.703127,0.701175,0.699223,0.69727,0.695317,0.693363,0.691409,0.689454,0.687499,0.685543,0.683588,0.681632,0.679675,0.677719,0.675762,0.673804,0.671847,0.669889,0.667932,0.665974,0.664016,0.662057,0.660099,0.658141,0.656182,0.654223,0.652265,0.650306,0.648347,0.646389,0.64443,0.642472,0.640513,0.638554,0.636596,0.634638,0.63268,0.630721,0.628763,0.626806,0.624848,0.622891,0.620933,0.618976,0.61702,0.615063,0.613107,0.611151,0.609195,0.607239,0.605284,0.603329,0.601375,0.59942,0.597467,0.595513,0.59356,0.591607,0.589655,0.587703,0.585751,0.5838,0.581849,0.579899,0.577949,0.575999,0.57405,0.572102,0.570154,0.568207,0.56626,0.564313,0.562367,0.560422,0.558477,0.556533,0.554589,0.552646,0.550703,0.548761,0.54682,0.544879,0.542939,0.540999,0.53906,0.537122,0.535184,0.533247,0.531311,0.529375,0.52744,0.525506,0.523572,0.521639,0.519707,0.517775,0.515844,0.513914,0.511985,0.510056,0.508128,0.506201,0.504274,0.502349,0.500424},
      {0,1.60035e-23,3.73626e-14,5.33442e-11,2.16432e-09,2.12651e-08,1.02932e-07,3.32104e-07,8.30066e-07,1.74665e-06,3.25252e-06,5.5331e-06,8.78423e-06,1.32091e-05,1.90162e-05,2.64177e-05,3.56289e-05,4.68668e-05,6.03501e-05,7.62989e-05,9.49341e-05,0.000116477,0.00014115,0.000169176,0.000200778,0.000236178,0.000275601,0.00031927,0.00036741,0.000420243,0.000477995,0.00054089,0.000609151,0.000683004,0.000762673,0.000848382,0.000940355,0.00103882,0.001144,0.00125611,0.00137539,0.00150206,0.00163634,0.00177845,0.00192863,0.0020871,0.00225408,0.0024298,0.00261447,0.00280834,0.00301161,0.00322452,0.00344729,0.00368015,0.00393391,0.00419109,0.00445073,0.00471272,0.00497696,0.00524336,0.00551182,0.00578227,0.00609506,0.00642463,0.00675656,0.00709075,0.00742713,0.0077656,0.00810608,0.00847465,0.00887094,0.0092695,0.00967025,0.0100731,0.010478,0.0108848,0.0113462,0.0118114,0.0122789,0.0127484,0.01322,0.0136935,0.0141995,0.0147089,0.0152203,0.0157337,0.0162488,0.0167845,0.0173549,0.0179274,0.0185018,0.019078,0.0196593,0.0202695,0.0208815,0.0214953,0.0221109,0.0227336,0.0233926,0.0240534,0.024716,0.0253803,0.0260579,0.0267541,0.0274519,0.0281514,0.0288525,0.0295816,0.0303156,0.0310512,0.0317883,0.0325419,0.0333089,0.0340774,0.0348475,0.035627,0.0364206,0.0372156,0.038012,0.0388191,0.0396434,0.0404692,0.0412964,0.0421309,0.0429734,0.0438173,0.0446624,0.0455253,0.0463965,0.0472689,0.0481429,0.0490236,0.0499055,0.0507886,0.0516834,0.0525908,0.0534993,0.0544089,0.0553126,0.0562173,0.0571232,0.058036,0.0589535,0.0598719,0.060802,0.0617549,0.0627089,0.0636644,0.0646252,0.0655871,0.06655,0.0674912,0.0684319,0.0693734,0.0703189,0.0712659,0.0722137,0.0731888,0.0741709,0.0751539,0.0761379,0.0771228,0.0781085,0.0790642,0.0800177,0.0809719,0.0819253,0.0828794,0.0836337,0.0836976,0.0837608,0.0838233,0.083885,0.0839462,0.0840066,0.0840664,0.0841255,0.084184,0.0842419,0.0842992,0.0843558,0.0844119,0.0844673,0.0845222,0.0845766,0.0846303,0.0846835,0.0847362,0.0847883,0.0848399,0.084891,0.0849416,0.0849917,0.0850412,0.0850903,0.0851389,0.085187,0.0852347,0.0852819,0.0853287,0.0853749,0.0854208,0.0854662,0.0855112,0.0855558,0.0855999,0.0856437,0.085687,0.0857299,0.0857724,0.0858146,0.0858563,0.0858977,0.0859387,0.0859794,0.0860196,0.0860596,0.0860991,0.0861383,0.0861772,0.0862157,0.0862539,0.0862918,0.0863293,0.0863665,0.0864034,0.0864399,0.0864762,0.0865122,0.0865478,0.0865832,0.0866182,0.086653,0.0866874,0.0867216,0.0867555,0.0867891,0.0868225,0.0868556,0.0868884,0.0869209,0.0869532,0.0869852,0.087017,0.0870485,0.0870798,0.0871108,0.0871416,0.0871721,0.0872024,0.0872325,0.0872623,0.0872919,0.0873213,0.0873504,0.0873794,0.0874081,0.0874365,0.0874648,0.0874929,0.0875207,0.0875484,0.0875758,0.087603,0.0876301,0.0876569,0.0876835,0.08771,0.0877362,0.0877623,0.0877881,0.0878138,0.0878393,0.0878646,0.0878898,0.0879147,0.0879395,0.0879641,0.0879886,0.0880128,0.0880369,0.0880608,0.0880846,0.0881082,0.0881316,0.0881549,0.088178,0.088201,0.0882238,0.0882464,0.0882689,0.0882912,0.0883134,0.0883355,0.0883573,0.0883791,0.0884007,0.0884222,0.0884435,0.0884647,0.0884857,0.0885066,0.0885274,0.088548,0.0885685,0.0885889,0.0886091,0.0886292,0.0886492,0.088669,0.0886888,0.0887084,0.0887278,0.0887472,0.0887664,0.0887855,0.0888045,0.0888234,0.0888422,0.0888608,0.0888793,0.0888977,0.0889161,0.0889342,0.0889523,0.0889703,0.0889881,0.0890059,0.0890235,0.0890411,0.0890585,0.0890759,0.0890931,0.0891102,0.0891272,0.0891442,0.089161,0.0891777,0.0891943,0.0892109,0.0892273,0.0892436,0.0892599,0.089276,0.0892921,0.0893081,0.0893239,0.0893397,0.0893554,0.089371,0.0893865,0.089402,0.0894173,0.0894326,0.0894477,0.0894628,0.0894778,0.0894927,0.0895076,0.0895223,0.089537,0.0895516,0.0895661,0.0895805,0.0895949,0.0896092,0.0896234,0.0896375,0.0896515,0.0896655,0.0896794,0.0896932,0.089707,0.0897206,0.0897342,0.0897478,0.0897612,0.0897746,0.0897879,0.0898012,0.0898143,0.0898275,0.0898405,0.0898535,0.0898664,0.0898792,0.089892,0.0899047,0.0899173,0.0899299,0.0899424,0.0899548,0.0899672,0.0899795,0.0899918,0.090004,0.0900161,0.0900282,0.0900402,0.0900522,0.0900641},
      {0,-9.5743e-24,-2.23526e-14,-3.19138e-11,-1.29483e-09,-1.27221e-08,-6.15803e-08,-1.98685e-07,-4.96596e-07,-1.04495e-06,-1.94586e-06,-3.31024e-06,-5.25526e-06,-7.90249e-06,-1.13766e-05,-1.58047e-05,-2.13154e-05,-2.80386e-05,-3.61051e-05,-4.56467e-05,-5.67953e-05,-6.96837e-05,-8.44447e-05,-0.000101211,-0.000120117,-0.000141296,-0.000164881,-0.000191007,-0.000219807,-0.000251415,-0.000285966,-0.000323593,-0.000364431,-0.000408615,-0.000456277,-0.000507553,-0.000562578,-0.000621484,-0.000684407,-0.000751482,-0.000822842,-0.000898622,-0.000978956,-0.00106398,-0.00115383,-0.00124863,-0.00134853,-0.00145365,-0.00156413,-0.00168011,-0.00180172,-0.0019291,-0.00206237,-0.00220168,-0.002351,-0.00250217,-0.00265478,-0.00280876,-0.00296405,-0.0031206,-0.00327835,-0.00343725,-0.00361969,-0.00381148,-0.00400463,-0.00419908,-0.00439479,-0.0045917,-0.00478977,-0.00500318,-0.00523174,-0.00546159,-0.00569268,-0.00592497,-0.00615841,-0.00639296,-0.00665615,-0.00692147,-0.00718801,-0.00745572,-0.00772456,-0.0079945,-0.00828289,-0.0085732,-0.00886461,-0.00915711,-0.00945064,-0.00975451,-0.0100759,-0.0103984,-0.010722,-0.0110466,-0.011374,-0.0117176,-0.0120623,-0.0124079,-0.0127545,-0.0131047,-0.013473,-0.0138422,-0.0142124,-0.0145835,-0.014962,-0.0153508,-0.0157405,-0.0161312,-0.0165227,-0.0169282,-0.0173364,-0.0177453,-0.0181551,-0.0185741,-0.0190004,-0.0194276,-0.0198557,-0.0202887,-0.0207291,-0.0211704,-0.0216124,-0.0220603,-0.0225177,-0.0229759,-0.0234349,-0.0238982,-0.0243664,-0.0248353,-0.0253048,-0.0257843,-0.0262683,-0.0267529,-0.0272385,-0.0277298,-0.0282217,-0.0287142,-0.0292133,-0.0297193,-0.030226,-0.0307334,-0.031242,-0.0317512,-0.032261,-0.0327758,-0.0332937,-0.0338122,-0.0343365,-0.0348721,-0.0354083,-0.0359456,-0.0364887,-0.0370324,-0.0375766,-0.0381167,-0.038657,-0.0391978,-0.0397433,-0.04029,-0.0408373,-0.0413974,-0.0419609,-0.042525,-0.0430931,-0.0436624,-0.0442322,-0.0447958,-0.0453592,-0.0459232,-0.0464907,-0.0470587,-0.0475073,-0.0475438,-0.0475798,-0.0476154,-0.0476506,-0.0476854,-0.0477198,-0.0477539,-0.0477876,-0.0478209,-0.0478539,-0.0478866,-0.0479188,-0.0479508,-0.0479824,-0.0480137,-0.0480446,-0.0480753,-0.0481056,-0.0481356,-0.0481653,-0.0481947,-0.0482239,-0.0482527,-0.0482812,-0.0483095,-0.0483374,-0.0483651,-0.0483926,-0.0484197,-0.0484466,-0.0484733,-0.0484997,-0.0485258,-0.0485517,-0.0485773,-0.0486027,-0.0486279,-0.0486528,-0.0486775,-0.048702,-0.0487262,-0.0487502,-0.0487741,-0.0487976,-0.048821,-0.0488442,-0.0488671,-0.0488899,-0.0489124,-0.0489348,-0.0489569,-0.0489789,-0.0490007,-0.0490222,-0.0490436,-0.0490648,-0.0490859,-0.0491067,-0.0491274,-0.0491479,-0.0491682,-0.0491883,-0.0492083,-0.0492281,-0.0492478,-0.0492673,-0.0492866,-0.0493058,-0.0493248,-0.0493436,-0.0493623,-0.0493809,-0.0493993,-0.0494175,-0.0494357,-0.0494536,-0.0494714,-0.0494891,-0.0495067,-0.0495241,-0.0495414,-0.0495585,-0.0495755,-0.0495924,-0.0496091,-0.0496257,-0.0496422,-0.0496586,-0.0496748,-0.0496909,-0.0497069,-0.0497228,-0.0497386,-0.0497542,-0.0497697,-0.0497851,-0.0498004,-0.0498156,-0.0498307,-0.0498457,-0.0498605,-0.0498753,-0.0498899,-0.0499045,-0.0499189,-0.0499332,-0.0499474,-0.0499616,-0.0499756,-0.0499895,-0.0500034,-0.0500171,-0.0500307,-0.0500443,-0.0500577,-0.0500711,-0.0500844,-0.0500975,-0.0501106,-0.0501236,-0.0501365,-0.0501494,-0.0501621,-0.0501747,-0.0501873,-0.0501998,-0.0502122,-0.0502245,-0.0502367,-0.0502489,-0.050261,-0.050273,-0.0502849,-0.0502967,-0.0503085,-0.0503202,-0.0503318,-0.0503433,-0.0503548,-0.0503662,-0.0503775,-0.0503888,-0.0503999,-0.050411,-0.0504221,-0.050433,-0.0504439,-0.0504548,-0.0504655,-0.0504762,-0.0504869,-0.0504974,-0.0505079,-0.0505184,-0.0505287,-0.050539,-0.0505493,-0.0505595,-0.0505696,-0.0505796,-0.0505896,-0.0505996,-0.0506095,-0.0506193,-0.0506291,-0.0506388,-0.0506484,-0.050658,-0.0506675,-0.050677,-0.0506865,-0.0506958,-0.0507051,-0.0507144,-0.0507236,-0.0507328,-0.0507419,-0.0507509,-0.0507599,-0.0507689,-0.0507778,-0.0507866,-0.0507954,-0.0508042,-0.0508129,-0.0508215,-0.0508301,-0.0508387,-0.0508472,-0.0508556,-0.0508641,-0.0508724,-0.0508807,-0.050889,-0.0508973,-0.0509054,-0.0509136,-0.0509217,-0.0509297,-0.0509377,-0.0509457,-0.0509536,-0.0509615,-0.0509693,-0.0509771,-0.0509849,-0.0509926,-0.0510003,-0.0510079,-0.0510155,-0.0510231,-0.0510306,-0.0510381,-0.0510455,-0.0510529,-0.0510602,-0.0510676,-0.0510748,-0.0510821,-0.0510893,-0.0510965,-0.0511036,-0.0511107,-0.0511178,-0.0511248,-0.0511318,-0.0511387,-0.0511457,-0.0511525,-0.0511594,-0.0511662,-0.051173},
      {0,2.00144e-24,4.67265e-15,6.67135e-12,2.70675e-10,2.65947e-09,1.28729e-08,4.15336e-08,1.0381e-07,2.1844e-07,4.06768e-07,6.91983e-07,1.09858e-06,1.65196e-06,2.37821e-06,3.30386e-06,4.45583e-06,5.86127e-06,7.54753e-06,9.54212e-06,1.18727e-05,1.45669e-05,1.76526e-05,2.11575e-05,2.51097e-05,2.9537e-05,3.44673e-05,3.99287e-05,4.59491e-05,5.25566e-05,5.97792e-05,6.76449e-05,7.61819e-05,8.54181e-05,9.53816e-05,0.000106101,0.000117603,0.000129917,0.000143071,0.000157092,0.000172009,0.000187851,0.000204644,0.000222417,0.000241199,0.000261017,0.0002819,0.000303875,0.000326971,0.000351216,0.000376637,0.000403264,0.000431124,0.000460245,0.000494743,0.000529872,0.000565351,0.000601165,0.000637301,0.000673743,0.000710479,0.000747498,0.000791949,0.000839309,0.000887026,0.000935086,0.000983476,0.00103218,0.00108119,0.00113562,0.0011954,0.00125555,0.00131604,0.00137687,0.00143803,0.0014995,0.00157383,0.00164895,0.00172446,0.00180033,0.00187655,0.00195312,0.00203502,0.00211751,0.00220034,0.0022835,0.00236698,0.00245662,0.00255696,0.00265769,0.00275878,0.00286025,0.00296263,0.00307017,0.00317807,0.00328632,0.00339492,0.00350604,0.00363162,0.00375758,0.00388392,0.00401062,0.00413994,0.00427292,0.00440625,0.00453994,0.00467397,0.00482272,0.00497364,0.00512494,0.00527661,0.00543177,0.00558978,0.00574816,0.00590689,0.00607215,0.00624754,0.00642331,0.00659945,0.00677801,0.00696051,0.00714338,0.0073266,0.00751759,0.00771717,0.00791713,0.00811745,0.00832212,0.00852883,0.0087359,0.00894396,0.00916696,0.00939034,0.00961408,0.00984091,0.010071,0.0103015,0.0105326,0.0107803,0.0110283,0.0112768,0.0115323,0.0117921,0.0120522,0.0123132,0.0125755,0.0128381,0.0131019,0.0133744,0.0136473,0.0139205,0.0142108,0.0145025,0.0147946,0.0150957,0.0153988,0.0157022,0.0160073,0.0163132,0.0166193,0.0169331,0.0172485,0.0175644,0.0178955,0.0182283,0.018562,0.0189051,0.0192486,0.0195184,0.0195334,0.0195482,0.0195629,0.0195774,0.0195917,0.0196059,0.0196199,0.0196337,0.0196474,0.019661,0.0196744,0.0196877,0.0197009,0.0197139,0.0197267,0.0197395,0.0197521,0.0197645,0.0197769,0.0197891,0.0198012,0.0198132,0.0198251,0.0198368,0.0198484,0.0198599,0.0198713,0.0198826,0.0198938,0.0199048,0.0199158,0.0199267,0.0199374,0.0199481,0.0199586,0.0199691,0.0199794,0.0199897,0.0199998,0.0200099,0.0200199,0.0200298,0.0200395,0.0200492,0.0200589,0.0200684,0.0200778,0.0200872,0.0200965,0.0201057,0.0201148,0.0201238,0.0201328,0.0201417,0.0201505,0.0201592,0.0201678,0.0201764,0.0201849,0.0201933,0.0202017,0.02021,0.0202182,0.0202264,0.0202344,0.0202425,0.0202504,0.0202583,0.0202661,0.0202739,0.0202816,0.0202892,0.0202968,0.0203043,0.0203117,0.0203191,0.0203265,0.0203337,0.020341,0.0203481,0.0203552,0.0203623,0.0203693,0.0203762,0.0203831,0.0203899,0.0203967,0.0204035,0.0204101,0.0204168,0.0204234,0.0204299,0.0204364,0.0204428,0.0204492,0.0204555,0.0204618,0.0204681,0.0204743,0.0204804,0.0204865,0.0204926,0.0204986,0.0205046,0.0205106,0.0205165,0.0205223,0.0205281,0.0205339,0.0205396,0.0205453,0.020551,0.0205566,0.0205622,0.0205677,0.0205732,0.0205786,0.0205841,0.0205895,0.0205948,0.0206001,0.0206054,0.0206106,0.0206158,0.020621,0.0206261,0.0206312,0.0206363,0.0206413,0.0206463,0.0206513,0.0206562,0.0206612,0.020666,0.0206709,0.0206757,0.0206805,0.0206852,0.0206899,0.0206946,0.0206993,0.0207039,0.0207085,0.0207131,0.0207176,0.0207221,0.0207266,0.0207311,0.0207355,0.0207399,0.0207443,0.0207486,0.0207529,0.0207572,0.0207615,0.0207657,0.0207699,0.0207741,0.0207783,0.0207824,0.0207866,0.0207906,0.0207947,0.0207987,0.0208028,0.0208068,0.0208107,0.0208147,0.0208186,0.0208225,0.0208264,0.0208302,0.0208341,0.0208379,0.0208417,0.0208454,0.0208492,0.0208529,0.0208566,0.0208603,0.020864,0.0208676,0.0208712,0.0208748,0.0208784,0.020882,0.0208855,0.020889,0.0208925,0.020896,0.0208995,0.0209029,0.0209063,0.0209097,0.0209131,0.0209165,0.0209198,0.0209232,0.0209265,0.0209298,0.0209331,0.0209363,0.0209396,0.0209428,0.020946,0.0209492,0.0209524,0.0209555,0.0209587,0.0209618,0.0209649,0.020968,0.0209711,0.0209741,0.0209772,0.0209802,0.0209832,0.0209862,0.0209892,0.0209921,0.0209951,0.020998,0.021001,0.0210039,0.0210067,0.0210096,0.0210125,0.0210153,0.0210182,0.021021,0.0210238,0.0210266}};
   double data_ddeltadq[4][401]={{0,78.5398,8.24658e-09,4.81692e-06,9.78069e-05,0.000540878,0.00158999,0.00329552,0.00553356,0.00811834,0.0108775,0.0136795,0.0164344,0.0190852,0.021598,0.0239545,0.0261455,0.0281678,0.0300213,0.0317076,0.0332292,0.0345893,0.0357909,0.0368375,0.0377327,0.0384803,0.0390846,0.0395501,0.0398819,0.0400854,0.0401666,0.0401321,0.0399885,0.0397431,0.0394035,0.0389773,0.0384722,0.0378962,0.0372571,0.0365623,0.0358194,0.0350354,0.0342171,0.033371,0.032503,0.0316187,0.0307231,0.0298209,0.0289163,0.0280131,0.0271146,0.0262238,0.0253432,0.024475,0.0236211,0.0227831,0.0219623,0.0211597,0.0203762,0.0196123,0.0188687,0.0181454,0.0174428,0.0167608,0.0160993,0.0154583,0.0148375,0.0142366,0.0136552,0.013093,0.0125495,0.0120243,0.0115169,0.0110269,0.0105537,0.0100968,0.00965579,0.00923008,0.0088192,0.00842265,0.00803995,0.0076706,0.00731415,0.00697013,0.0066381,0.00631762,0.00600826,0.00570961,0.00542129,0.0051429,0.00487406,0.00461443,0.00436366,0.00412141,0.00388736,0.0036612,0.00344263,0.00323136,0.00302713,0.00282967,0.00263871,0.00245402,0.00227536,0.00210251,0.00193525,0.00177336,0.00161667,0.00146496,0.00131806,0.00117578,0.00103797,0.000904461,0.000775093,0.000649716,0.000528189,0.000410372,0.000296138,0.000185352,7.78974e-05,-2.63445e-05,-0.000127486,-0.000225635,-0.000320896,-0.000413369,-0.000503149,-0.000590327,-0.000674994,-0.000757234,-0.000837128,-0.000914755,-0.000990191,-0.00106351,-0.00113478,-0.00120407,-0.00127144,-0.00133695,-0.00140068,-0.00146266,-0.00152297,-0.00158165,-0.00163875,-0.00169433,-0.00174842,-0.00180109,-0.00185237,-0.0019023,-0.00195093,-0.0019983,-0.00204444,-0.0020894,-0.0021332,-0.00217589,-0.00221749,-0.00225804,-0.00229756,-0.0023361,-0.00237367,-0.00241032,-0.00244605,-0.0024809,-0.0025149,-0.00254807,-0.00258043,-0.002612,-0.00264281,-0.00267288,-0.00270223,-0.00273088,-0.00275885,-0.00278615,-0.00281281,-0.00283884,-0.00286427,-0.00288909,-0.00291334,-0.00293703,-0.00296017,-0.00298278,-0.00300486,-0.00302644,-0.00304753,-0.00306814,-0.00308828,-0.00310797,-0.00312721,-0.00314602,-0.0031644,-0.00318238,-0.00319995,-0.00321713,-0.00323393,-0.00325036,-0.00326642,-0.00328213,-0.00329749,-0.00331252,-0.00332722,-0.00334159,-0.00335565,-0.00336941,-0.00338286,-0.00339602,-0.0034089,-0.00342149,-0.00343382,-0.00344587,-0.00345767,-0.00346921,-0.0034805,-0.00349155,-0.00350236,-0.00351293,-0.00352328,-0.0035334,-0.00354331,-0.003553,-0.00356249,-0.00357177,-0.00358085,-0.00358973,-0.00359843,-0.00360693,-0.00361525,-0.00362339,-0.00363136,-0.00363915,-0.00364677,-0.00365423,-0.00366153,-0.00366866,-0.00367564,-0.00368247,-0.00368915,-0.00369568,-0.00370207,-0.00370831,-0.00371442,-0.00372039,-0.00372623,-0.00373194,-0.00373752,-0.00374298,-0.00374831,-0.00375353,-0.00375862,-0.0037636,-0.00376846,-0.00377322,-0.00377786,-0.0037824,-0.00378683,-0.00379115,-0.00379538,-0.0037995,-0.00380353,-0.00380746,-0.00381129,-0.00381504,-0.00381869,-0.00382225,-0.00382572,-0.00382911,-0.00383241,-0.00383563,-0.00383877,-0.00384183,-0.0038448,-0.0038477,-0.00385053,-0.00385328,-0.00385595,-0.00385855,-0.00386108,-0.00386354,-0.00386594,-0.00386826,-0.00387052,-0.00387271,-0.00387483,-0.0038769,-0.0038789,-0.00388083,-0.00388271,-0.00388453,-0.00388629,-0.00388799,-0.00388964,-0.00389123,-0.00389276,-0.00389424,-0.00389567,-0.00389704,-0.00389836,-0.00389963,-0.00390085,-0.00390202,-0.00390314,-0.00390422,-0.00390525,-0.00390623,-0.00390716,-0.00390805,-0.00390889,-0.00390969,-0.00391045,-0.00391116,-0.00391183,-0.00391246,-0.00391305,-0.0039136,-0.00391411,-0.00391458,-0.00391501,-0.0039154,-0.00391576,-0.00391608,-0.00391636,-0.0039166,-0.00391681,-0.00391699,-0.00391713,-0.00391723,-0.00391731,-0.00391735,-0.00391735,-0.00391733,-0.00391727,-0.00391718,-0.00391706,-0.00391692,-0.00391674,-0.00391653,-0.00391629,-0.00391602,-0.00391573,-0.0039154,-0.00391505,-0.00391467,-0.00391427,-0.00391384,-0.00391338,-0.0039129,-0.00391239,-0.00391186,-0.0039113,-0.00391072,-0.00391011,-0.00390948,-0.00390883,-0.00390815,-0.00390745,-0.00390673,-0.00390599,-0.00390522,-0.00390443,-0.00390363,-0.0039028,-0.00390195,-0.00390108,-0.00390019,-0.00389928,-0.00389835,-0.0038974,-0.00389643,-0.00389544,-0.00389444,-0.00389341,-0.00389237,-0.00389131,-0.00389023,-0.00388914,-0.00388803,-0.0038869,-0.00388575,-0.00388459,-0.00388341,-0.00388221,-0.003881,-0.00387977,-0.00387853,-0.00387727,-0.003876,-0.00387471,-0.0038734,-0.00387209,-0.00387075,-0.00386941,-0.00386804,-0.00386667,-0.00386528,-0.00386388,-0.00386246,-0.00386103,-0.00385959,-0.00385813,-0.00385666,-0.00385518,-0.00385368,-0.00385218,-0.00385066,-0.00384913},
      {0,1.3783e-21,8.09557e-13,5.21435e-10,1.21898e-08,7.91177e-08,2.76009e-07,6.81396e-07,1.36068e-06,2.36279e-06,3.72272e-06,5.46512e-06,7.6074e-06,1.01619e-05,1.31376e-05,1.6541e-05,2.03768e-05,2.46488e-05,2.93595e-05,3.45112e-05,4.01055e-05,4.61436e-05,5.26266e-05,5.95553e-05,6.69303e-05,7.47522e-05,8.30214e-05,9.17384e-05,0.000100903,0.000110516,0.000120578,0.000131088,0.000142048,0.000153456,0.000165313,0.000177619,0.000190374,0.000203579,0.000217233,0.000231336,0.000245889,0.000260892,0.000276344,0.000292246,0.000308597,0.000325398,0.000342648,0.000360348,0.000378498,0.000397098,0.000416148,0.000435647,0.000455596,0.000475995,0.000511864,0.000516903,0.000521719,0.000526327,0.000530737,0.000534961,0.000539008,0.000542889,0.000656794,0.000661614,0.000666242,0.00067069,0.000674966,0.000679078,0.000683036,0.00079038,0.000794998,0.000799449,0.000803741,0.00080788,0.000811874,0.000815729,0.00092848,0.000932894,0.00093716,0.000941284,0.000945273,0.000949132,0.00101701,0.00102104,0.00102494,0.00102872,0.00103239,0.00113913,0.00114317,0.00114709,0.0011509,0.00115459,0.00121863,0.00122243,0.00122612,0.0012297,0.00123318,0.00131651,0.00132018,0.00132375,0.00132722,0.0013306,0.00139092,0.00139436,0.00139772,0.00140099,0.00140417,0.00146669,0.00146994,0.00147312,0.00147621,0.00153285,0.00153599,0.00153905,0.00154204,0.00158605,0.00158904,0.00159195,0.0015948,0.0016478,0.00165067,0.00165349,0.00165624,0.00168417,0.00168687,0.00168952,0.0016921,0.00174159,0.0017442,0.00174675,0.00176076,0.00176323,0.00176566,0.00176804,0.00181417,0.00181655,0.0018189,0.00180691,0.00180912,0.0018113,0.00181343,0.00183454,0.00183663,0.00183868,0.00190558,0.0019077,0.00190977,0.00192153,0.00192355,0.00192553,0.00192747,0.00188127,0.00188306,0.00188481,0.00189391,0.00189562,0.00189729,0.00196434,0.00196608,0.00196778,0.00196989,0.00197154,0.00197316,0.00190727,0.00190872,0.0019072,0.00190861,0.00190999,0.000129533,0.000128095,0.000126681,0.000125291,0.000123923,0.000122577,0.000121254,0.000119951,0.00011867,0.000117409,0.000116169,0.000114947,0.000113745,0.000112562,0.000111398,0.000110251,0.000109122,0.00010801,0.000106916,0.000105837,0.000104776,0.00010373,0.0001027,0.000101685,0.000100685,9.96997e-05,9.8729e-05,9.77725e-05,9.68298e-05,9.59009e-05,9.49852e-05,9.40827e-05,9.3193e-05,9.2316e-05,9.14513e-05,9.05988e-05,8.97582e-05,8.89293e-05,8.81119e-05,8.73058e-05,8.65107e-05,8.57265e-05,8.4953e-05,8.41899e-05,8.34372e-05,8.26945e-05,8.19618e-05,8.12388e-05,8.05254e-05,7.98214e-05,7.91266e-05,7.8441e-05,7.77642e-05,7.70963e-05,7.64369e-05,7.5786e-05,7.51435e-05,7.45091e-05,7.38828e-05,7.32644e-05,7.26538e-05,7.20509e-05,7.14554e-05,7.08674e-05,7.02866e-05,6.9713e-05,6.91465e-05,6.85868e-05,6.8034e-05,6.74879e-05,6.69484e-05,6.64153e-05,6.58887e-05,6.53683e-05,6.48542e-05,6.43461e-05,6.3844e-05,6.33478e-05,6.28574e-05,6.23727e-05,6.18937e-05,6.14202e-05,6.09522e-05,6.04895e-05,6.00321e-05,5.958e-05,5.91329e-05,5.86909e-05,5.82539e-05,5.78218e-05,5.73946e-05,5.69721e-05,5.65543e-05,5.61411e-05,5.57324e-05,5.53283e-05,5.49285e-05,5.45331e-05,5.4142e-05,5.37552e-05,5.33725e-05,5.29939e-05,5.26194e-05,5.22489e-05,5.18823e-05,5.15196e-05,5.11607e-05,5.08056e-05,5.04542e-05,5.01065e-05,4.97624e-05,4.94219e-05,4.90849e-05,4.87513e-05,4.84212e-05,4.80945e-05,4.77711e-05,4.74509e-05,4.71341e-05,4.68204e-05,4.65099e-05,4.62025e-05,4.58981e-05,4.55968e-05,4.52985e-05,4.50031e-05,4.47107e-05,4.44211e-05,4.41344e-05,4.38504e-05,4.35693e-05,4.32908e-05,4.30151e-05,4.2742e-05,4.24715e-05,4.22036e-05,4.19383e-05,4.16755e-05,4.14152e-05,4.11573e-05,4.09019e-05,4.06489e-05,4.03983e-05,4.015e-05,3.9904e-05,3.96603e-05,3.94188e-05,3.91796e-05,3.89426e-05,3.87077e-05,3.8475e-05,3.82445e-05,3.8016e-05,3.77896e-05,3.75652e-05,3.73428e-05,3.71225e-05,3.69041e-05,3.66877e-05,3.64732e-05,3.62606e-05,3.60499e-05,3.5841e-05,3.5634e-05,3.54287e-05,3.52253e-05,3.50237e-05,3.48238e-05,3.46256e-05,3.44292e-05,3.42344e-05,3.40414e-05,3.38499e-05,3.36601e-05,3.3472e-05,3.32854e-05,3.31004e-05,3.2917e-05,3.27351e-05,3.25547e-05,3.23759e-05,3.21985e-05,3.20226e-05,3.18482e-05,3.16753e-05,3.15037e-05,3.13336e-05,3.11649e-05,3.09975e-05,3.08315e-05,3.06669e-05,3.05036e-05,3.03416e-05,3.0181e-05,3.00216e-05,2.98635e-05,2.97067e-05,2.95512e-05,2.93969e-05,2.92438e-05,2.90919e-05,2.89412e-05,2.87917e-05,2.86434e-05,2.84963e-05,2.83503e-05,2.82054e-05,2.80617e-05,2.79191e-05,2.77776e-05,2.76372e-05,2.74979e-05,2.73596e-05,2.72224e-05,2.70863e-05,2.69512e-05,2.68171e-05,2.66841e-05,2.6552e-05,2.6421e-05,2.62909e-05,2.61618e-05,2.60337e-05,2.59065e-05,2.57803e-05,2.56551e-05,2.55307e-05,2.54073e-05,2.52848e-05,2.51632e-05,2.50425e-05,2.49226e-05,2.48037e-05},
      {0,-8.24586e-22,-4.84327e-13,-3.11954e-10,-7.29269e-09,-4.73331e-08,-1.65125e-07,-4.07652e-07,-8.14042e-07,-1.41357e-06,-2.22716e-06,-3.26957e-06,-4.55121e-06,-6.07949e-06,-7.85973e-06,-9.89584e-06,-1.21907e-05,-1.47464e-05,-1.75647e-05,-2.06467e-05,-2.39936e-05,-2.76059e-05,-3.14844e-05,-3.56296e-05,-4.00418e-05,-4.47213e-05,-4.96685e-05,-5.48835e-05,-6.03665e-05,-6.61177e-05,-7.21372e-05,-7.84251e-05,-8.49814e-05,-9.18064e-05,-9.89e-05,-0.000106262,-0.000113893,-0.000121793,-0.000129962,-0.000138399,-0.000147106,-0.000156081,-0.000165325,-0.000174839,-0.000184621,-0.000194672,-0.000204992,-0.000215581,-0.00022644,-0.000237567,-0.000248963,-0.000260629,-0.000272563,-0.000284767,-0.000300901,-0.000303835,-0.000306641,-0.000309325,-0.000311894,-0.000314354,-0.000316711,-0.000318971,-0.000382238,-0.00038501,-0.000387672,-0.00039023,-0.000392689,-0.000395054,-0.00039733,-0.000455865,-0.000458491,-0.000461022,-0.000463462,-0.000465815,-0.000468086,-0.000470278,-0.000529491,-0.000531963,-0.000534352,-0.000536662,-0.000538895,-0.000541056,-0.000579586,-0.000581845,-0.000584034,-0.000586153,-0.000588208,-0.000641841,-0.000644069,-0.000646232,-0.000648329,-0.000650366,-0.000686302,-0.000688398,-0.000690434,-0.000692412,-0.000694335,-0.000735655,-0.000737656,-0.000739602,-0.000741496,-0.00074334,-0.000776848,-0.000778728,-0.000780559,-0.000782342,-0.00078408,-0.000815538,-0.000817302,-0.000819021,-0.000820699,-0.000852104,-0.000853806,-0.000855466,-0.000857086,-0.000880364,-0.00088198,-0.000883558,-0.000885099,-0.00091437,-0.000915927,-0.000917448,-0.000918936,-0.000935874,-0.000937341,-0.000938775,-0.000940177,-0.000967596,-0.00096901,-0.000970392,-0.000982128,-0.000983479,-0.000984803,-0.000986099,-0.00101182,-0.00101312,-0.0010144,-0.00101701,-0.00101824,-0.00101944,-0.00102062,-0.0010357,-0.00103687,-0.00103801,-0.00107104,-0.0010722,-0.00107335,-0.0010861,-0.00108722,-0.00108833,-0.00108941,-0.00108062,-0.00108165,-0.00108265,-0.00109347,-0.00109445,-0.00109543,-0.0011271,-0.00112809,-0.00112906,-0.00113871,-0.00113967,-0.00114061,-0.00112689,-0.00112777,-0.00113526,-0.00113613,-0.00113698,-7.38132e-05,-7.29943e-05,-7.21891e-05,-7.13971e-05,-7.06182e-05,-6.98519e-05,-6.90981e-05,-6.83565e-05,-6.76268e-05,-6.69087e-05,-6.6202e-05,-6.55065e-05,-6.4822e-05,-6.41482e-05,-6.34848e-05,-6.28317e-05,-6.21887e-05,-6.15555e-05,-6.0932e-05,-6.0318e-05,-5.97132e-05,-5.91175e-05,-5.85307e-05,-5.79526e-05,-5.73831e-05,-5.6822e-05,-5.62691e-05,-5.57242e-05,-5.51873e-05,-5.46581e-05,-5.41366e-05,-5.36225e-05,-5.31157e-05,-5.26161e-05,-5.21236e-05,-5.16379e-05,-5.11591e-05,-5.06869e-05,-5.02213e-05,-4.9762e-05,-4.93091e-05,-4.88624e-05,-4.84217e-05,-4.7987e-05,-4.75582e-05,-4.71351e-05,-4.67177e-05,-4.63058e-05,-4.58994e-05,-4.54983e-05,-4.51025e-05,-4.47119e-05,-4.43263e-05,-4.39458e-05,-4.35701e-05,-4.31993e-05,-4.28332e-05,-4.24718e-05,-4.2115e-05,-4.17627e-05,-4.14148e-05,-4.10713e-05,-4.0732e-05,-4.0397e-05,-4.00661e-05,-3.97393e-05,-3.94165e-05,-3.90976e-05,-3.87826e-05,-3.84715e-05,-3.81641e-05,-3.78604e-05,-3.75603e-05,-3.72638e-05,-3.69708e-05,-3.66813e-05,-3.63952e-05,-3.61125e-05,-3.58331e-05,-3.55569e-05,-3.5284e-05,-3.50142e-05,-3.47475e-05,-3.44838e-05,-3.42232e-05,-3.39656e-05,-3.37108e-05,-3.3459e-05,-3.321e-05,-3.29637e-05,-3.27203e-05,-3.24795e-05,-3.22414e-05,-3.2006e-05,-3.17731e-05,-3.15428e-05,-3.1315e-05,-3.10897e-05,-3.08668e-05,-3.06464e-05,-3.04283e-05,-3.02125e-05,-2.99991e-05,-2.9788e-05,-2.95791e-05,-2.93724e-05,-2.91678e-05,-2.89655e-05,-2.87652e-05,-2.85671e-05,-2.8371e-05,-2.81769e-05,-2.79848e-05,-2.77948e-05,-2.76066e-05,-2.74204e-05,-2.72361e-05,-2.70537e-05,-2.68731e-05,-2.66943e-05,-2.65174e-05,-2.63422e-05,-2.61687e-05,-2.5997e-05,-2.5827e-05,-2.56587e-05,-2.5492e-05,-2.53269e-05,-2.51635e-05,-2.50017e-05,-2.48414e-05,-2.46827e-05,-2.45256e-05,-2.43699e-05,-2.42158e-05,-2.40631e-05,-2.39119e-05,-2.37621e-05,-2.36137e-05,-2.34668e-05,-2.33212e-05,-2.3177e-05,-2.30342e-05,-2.28926e-05,-2.27524e-05,-2.26135e-05,-2.24759e-05,-2.23396e-05,-2.22045e-05,-2.20706e-05,-2.1938e-05,-2.18066e-05,-2.16763e-05,-2.15473e-05,-2.14194e-05,-2.12926e-05,-2.11671e-05,-2.10426e-05,-2.09192e-05,-2.07969e-05,-2.06758e-05,-2.05557e-05,-2.04366e-05,-2.03186e-05,-2.02016e-05,-2.00857e-05,-1.99707e-05,-1.98568e-05,-1.97439e-05,-1.96319e-05,-1.95209e-05,-1.94108e-05,-1.93017e-05,-1.91935e-05,-1.90862e-05,-1.89799e-05,-1.88744e-05,-1.87699e-05,-1.86662e-05,-1.85634e-05,-1.84614e-05,-1.83604e-05,-1.82601e-05,-1.81607e-05,-1.80621e-05,-1.79643e-05,-1.78673e-05,-1.77711e-05,-1.76757e-05,-1.75811e-05,-1.74873e-05,-1.73942e-05,-1.73019e-05,-1.72103e-05,-1.71194e-05,-1.70293e-05,-1.69399e-05,-1.68513e-05,-1.67633e-05,-1.6676e-05,-1.65894e-05,-1.65036e-05,-1.64183e-05,-1.63338e-05,-1.62499e-05,-1.61667e-05,-1.60841e-05,-1.60022e-05,-1.59209e-05,-1.58402e-05,-1.57602e-05,-1.56808e-05,-1.5602e-05,-1.55237e-05,-1.54461e-05,-1.53691e-05,-1.52927e-05,-1.52168e-05,-1.51415e-05,-1.50668e-05,-1.49927e-05,-1.49191e-05,-1.48461e-05,-1.47736e-05,-1.47016e-05,-1.46302e-05,-1.45593e-05,-1.4489e-05,-1.44191e-05,-1.43498e-05,-1.4281e-05,-1.42127e-05,-1.41448e-05},
      {0,1.72374e-22,1.01245e-13,6.52119e-11,1.52449e-09,9.89465e-09,3.45183e-08,8.52169e-08,1.7017e-07,2.95496e-07,4.65572e-07,6.83481e-07,9.51399e-07,1.27087e-06,1.64302e-06,2.06866e-06,2.54838e-06,3.08263e-06,3.67177e-06,4.31606e-06,5.01569e-06,5.77083e-06,6.5816e-06,7.44812e-06,8.37046e-06,9.34868e-06,1.03829e-05,1.1473e-05,1.26192e-05,1.38214e-05,1.50798e-05,1.63942e-05,1.77648e-05,1.91915e-05,2.06743e-05,2.22134e-05,2.38086e-05,2.546e-05,2.71676e-05,2.89314e-05,3.07514e-05,3.26276e-05,3.45601e-05,3.65487e-05,3.85936e-05,4.06947e-05,4.28521e-05,4.50657e-05,4.73355e-05,4.96616e-05,5.20439e-05,5.44824e-05,5.69772e-05,5.95282e-05,6.99024e-05,7.06199e-05,7.13058e-05,7.19619e-05,7.259e-05,7.31915e-05,7.37678e-05,7.43205e-05,9.43631e-05,9.50925e-05,9.57931e-05,9.64662e-05,9.71134e-05,9.77359e-05,9.8335e-05,0.000119201,0.000119943,0.000120657,0.000121346,0.00012201,0.000122651,0.000123269,0.000149882,0.000150654,0.000151399,0.00015212,0.000152818,0.000153492,0.000164643,0.000165342,0.00016602,0.000166676,0.000167313,0.000200315,0.0002011,0.000201861,0.000202599,0.000203316,0.000214755,0.000215486,0.000216196,0.000216887,0.000217557,0.000250804,0.00025159,0.000252353,0.000253097,0.00025382,0.000265633,0.000266365,0.000267078,0.000267773,0.000268449,0.00030152,0.000302285,0.000303031,0.000303758,0.000315737,0.000316469,0.000317183,0.00031788,0.000350471,0.00035124,0.000351991,0.000352725,0.000364722,0.000365457,0.000366175,0.000366877,0.00039888,0.000399641,0.000400385,0.000401112,0.000413158,0.000413885,0.000414596,0.000445739,0.000446498,0.000447242,0.00044797,0.000460017,0.000460743,0.000461454,0.000495147,0.000495905,0.000496649,0.000497378,0.00051937,0.000520117,0.000520849,0.000524319,0.00052503,0.000525726,0.000544829,0.000545537,0.000546231,0.000546912,0.000583276,0.000584,0.000584711,0.000605982,0.000606705,0.000607414,0.000611507,0.000612197,0.000612876,0.000630846,0.00063153,0.000632202,0.000665586,0.000666288,0.000686198,0.000686908,0.000687606,3.03642e-05,3.00274e-05,2.96963e-05,2.93706e-05,2.90502e-05,2.87351e-05,2.84251e-05,2.812e-05,2.78199e-05,2.75246e-05,2.7234e-05,2.69479e-05,2.66664e-05,2.63893e-05,2.61164e-05,2.58478e-05,2.55834e-05,2.5323e-05,2.50665e-05,2.4814e-05,2.45652e-05,2.43202e-05,2.40789e-05,2.38411e-05,2.36069e-05,2.33761e-05,2.31487e-05,2.29246e-05,2.27038e-05,2.24861e-05,2.22716e-05,2.20601e-05,2.18517e-05,2.16462e-05,2.14436e-05,2.12439e-05,2.10469e-05,2.08527e-05,2.06612e-05,2.04723e-05,2.0286e-05,2.01023e-05,1.9921e-05,1.97422e-05,1.95658e-05,1.93918e-05,1.92201e-05,1.90507e-05,1.88835e-05,1.87185e-05,1.85557e-05,1.83951e-05,1.82365e-05,1.80799e-05,1.79254e-05,1.77729e-05,1.76223e-05,1.74737e-05,1.73269e-05,1.7182e-05,1.70389e-05,1.68975e-05,1.6758e-05,1.66202e-05,1.64841e-05,1.63496e-05,1.62169e-05,1.60857e-05,1.59561e-05,1.58281e-05,1.57017e-05,1.55768e-05,1.54533e-05,1.53314e-05,1.52109e-05,1.50918e-05,1.49741e-05,1.48578e-05,1.47428e-05,1.46292e-05,1.4517e-05,1.4406e-05,1.42963e-05,1.41878e-05,1.40806e-05,1.39746e-05,1.38698e-05,1.37662e-05,1.36638e-05,1.35625e-05,1.34624e-05,1.33633e-05,1.32654e-05,1.31685e-05,1.30727e-05,1.2978e-05,1.28843e-05,1.27916e-05,1.26999e-05,1.26092e-05,1.25195e-05,1.24308e-05,1.2343e-05,1.22561e-05,1.21702e-05,1.20851e-05,1.2001e-05,1.19177e-05,1.18354e-05,1.17539e-05,1.16732e-05,1.15933e-05,1.15143e-05,1.14361e-05,1.13588e-05,1.12822e-05,1.12063e-05,1.11313e-05,1.1057e-05,1.09834e-05,1.09106e-05,1.08386e-05,1.07672e-05,1.06966e-05,1.06266e-05,1.05574e-05,1.04888e-05,1.04209e-05,1.03537e-05,1.02871e-05,1.02212e-05,1.01559e-05,1.00912e-05,1.00272e-05,9.96379e-06,9.90098e-06,9.83877e-06,9.77715e-06,9.71612e-06,9.65566e-06,9.59577e-06,9.53645e-06,9.47767e-06,9.41945e-06,9.36177e-06,9.30463e-06,9.24801e-06,9.19192e-06,9.13634e-06,9.08127e-06,9.02671e-06,8.97264e-06,8.91906e-06,8.86597e-06,8.81336e-06,8.76122e-06,8.70954e-06,8.65833e-06,8.60758e-06,8.55728e-06,8.50743e-06,8.45801e-06,8.40903e-06,8.36048e-06,8.31236e-06,8.26466e-06,8.21737e-06,8.1705e-06,8.12403e-06,8.07796e-06,8.03229e-06,7.98701e-06,7.94211e-06,7.89761e-06,7.85348e-06,7.80972e-06,7.76634e-06,7.72332e-06,7.68067e-06,7.63837e-06,7.59643e-06,7.55483e-06,7.51359e-06,7.47268e-06,7.43212e-06,7.39189e-06,7.35199e-06,7.31241e-06,7.27317e-06,7.23424e-06,7.19563e-06,7.15733e-06,7.11935e-06,7.08167e-06,7.0443e-06,7.00722e-06,6.97044e-06,6.93396e-06,6.89777e-06,6.86186e-06,6.82624e-06,6.7909e-06,6.75584e-06,6.72106e-06,6.68655e-06,6.65231e-06,6.61833e-06,6.58463e-06,6.55118e-06,6.51799e-06,6.48506e-06,6.45238e-06,6.41995e-06,6.38778e-06,6.35585e-06,6.32416e-06,6.29271e-06,6.2615e-06,6.23053e-06,6.19979e-06,6.16928e-06,6.13901e-06,6.10896e-06,6.07913e-06,6.04953e-06,6.02014e-06,5.99098e-06,5.96203e-06,5.93329e-06,5.90477e-06,5.87646e-06,5.84835e-06,5.82045e-06}};
//
  int iqdata,iq,ichannel;
  double w1,w2,q;
  for(ichannel=0;ichannel<4;ichannel++){
    for(iq=0;iq<nqmax;iq++){
      q=qarray[iq];
      iqdata=int(floor(q)/delqdata);
      if(iqdata<nqdata){
	w1=(delqdata*double(iqdata+1)-q)/delqdata;
	w2=1.0-w1;
	delta[ichannel][iq]=w1*data_delta[ichannel][iqdata]
	  +w2*data_delta[ichannel][iqdata+1];
	ddeltadq[ichannel][iq]=w1*data_ddeltadq[ichannel][iqdata]
	  +w2*data_ddeltadq[ichannel][iqdata+1];
      }
      else{
	delta[ichannel][iq]=0.0;
	ddeltadq[ichannel][iq]=0.0;
      }
      //if(ichannel>0) delta[ichannel][iq]=ddeltadq[ichannel][iq]=0.0;
    }
  }
}

#endif
