#ifndef __CWAVEFUNCTION_WF_PKPLUS_PHASESHIFT_CC__
#define __CWAVEFUNCTION_WF_PKPLUS_PHASESHIFT_CC__
#include "wavefunction.h"

CWaveFunction_pkplus_phaseshift::CWaveFunction_pkplus_phaseshift(string  parsfilename) : CWaveFunction(){
	int iq,ichannel;
	double q;
	ParsInit(parsfilename);

	m1=MPROTON;
	m2=MKAON;
	IDENTICAL=0;
	q1q2=1;
	if(COULOMB==0) q1q2=0;
	nchannels=3;
	ellmax=1;
	InitArrays();

	ell[0]=0;
	ell[1]=1;
	ell[2]=1;

	InitWaves();

	channelweight[0]=1.0;
	channelweight[1]=1.0;
	channelweight[2]=2.0;

	read_phaseshifts();

	for(ichannel=0;ichannel<nchannels;ichannel++){
		for(iq=0;iq<nqmax;iq++){
			//if(ichannel>=0){
			//delta[ichannel][iq]=ddeltadq[ichannel][iq]=0.0;
			//}
			q=qarray[iq];
			/* printf("ichannel=%d, q=%g, delta=%g, ddeltadq=%g\n",
			ichannel,q,delta[ichannel][iq]*180.0/PI,
				ddeltadq[ichannel][iq]*180.0/PI); */
				Wepsilon[ichannel][iq]=ddeltadq[ichannel][iq]
				-GetIW(ell[ichannel],epsilon,q,q1q2,eta[iq],delta[ichannel][iq])
				+GetIW(ell[ichannel],epsilon,q,q1q2,eta[iq],0.0);
			Wepsilon[ichannel][iq]=3.0*Wepsilon[ichannel][iq]
				/(4.0*PI*pow(epsilon,3));
		}
	}
}

double CWaveFunction_pkplus_phaseshift::CalcPsiSquared(int iq,double r,double ctheta){
	double q,psisquared,x,dpsi2;
	double delta_s11,delta_p11,delta_p13;
	complex<double> psi,hstar0,hstar1,psi0;
	int ichannel;

	q=qarray[iq];
	x=q*r/HBARC;
	psi0=planewave[iq]->planewave(r,ctheta);
	if(iq>=nqmax){
		printf("iq too large!\n");
		exit(1);
	}

	if(r<epsilon){
		psisquared=real(psi0*conj(psi0));
		if(STRONG==1){
			for(ichannel=0;ichannel<nchannels;ichannel++){
				dpsi2=channelweight[ichannel]*2.0*PI*Wepsilon[ichannel][iq]
					*pow(HBARC,3)/(q*q);
				psisquared+=dpsi2;
			}
		}
	}
	else{
		if(STRONG==1){
			hstar0=partwave[0][iq]->GetPhiIncoming(r)/x;
			hstar1=partwave[1][iq]->GetPhiIncoming(r)/x;

			delta_s11=delta[0][iq];
			delta_p11=delta[1][iq];
			delta_p13=delta[2][iq];

			// s-wave m_s=+1/2
			psi=psi0+0.5*hstar0*(Misc::ceiphi(-2.0*delta_s11)-1.0);

			// m_s still equals 1/2, but now ell=1
			psi+=0.5*hstar1*((2.0/3.0)*Misc::ceiphi(-2.0*delta_p13)
				+(1.0/3.0)*Misc::ceiphi(-2.0*delta_p11)-1.0)*ci*(3.0)*ctheta;

			psisquared=real(psi*conj(psi));

			//Now consider m_s=-1/2, m_ell=1, Note: doesn't interfere with m_s=+1/2
			psi=0.5*hstar1*(Misc::ceiphi(-2.0*delta_p13)-Misc::ceiphi(-2.0*delta_p11))
				*ci*(3.0)*sqrt(1.0-ctheta*ctheta)/3.0;
			psisquared+=real(psi*conj(psi));
		}
		else{
			psi=psi0;
			psisquared=real(psi0*conj(psi0));
		}
	}
	if(psisquared>500.0){
		printf("_____________________________________________\n");
		printf("in CalcPsiSquared, q=%g,r=%g, psisquared=huge=%g\n",q,r,psisquared);
		printf("hstar0=(%g,%g), hstar1=(%g,%g),psi=(%g,%g)\n",real(hstar0),imag(hstar0),
			real(hstar1),imag(hstar1),real(psi),imag(psi));
	}
	psisquared*=RelativisticCorrection(r,iq);
	return psisquared;

}

void CWaveFunction_pkplus_phaseshift::read_phaseshifts(){
	int iq,iqdata,ichannel;
	double q,w1,w2;
//#include "pk_phaseshiftdat.cc"
	double delqdata=0.5;
	int nqdata=700;
	double data_delta[3][701]={{0,-3.20959e-15,-8.86676e-09,-1.2442e-06,-1.47461e-05,-6.51176e-05,-0.000175821,-0.000358928,-0.000615913,-0.000941989,-0.0013297,-0.00177103,-0.00225841,-0.00278516,-0.00334558,-0.00393477,-0.00454862,-0.00518372,-0.0058372,-0.00650665,-0.00719005,-0.00788567,-0.00859208,-0.00930801,-0.0100324,-0.0107644,-0.0115031,-0.0122539,-0.013012,-0.0137755,-0.0145437,-0.0153162,-0.0160928,-0.016873,-0.0176565,-0.0184431,-0.0192325,-0.0200245,-0.020819,-0.0216157,-0.0224021,-0.023185,-0.0239693,-0.024755,-0.0255419,-0.0263301,-0.0271193,-0.0279096,-0.0287009,-0.0294931,-0.0302861,-0.03108,-0.0318746,-0.0326765,-0.0334833,-0.0342908,-0.0350991,-0.035908,-0.0367176,-0.0375279,-0.0383387,-0.03915,-0.0399619,-0.0407743,-0.0415872,-0.0424006,-0.0432149,-0.04403,-0.0448455,-0.0456614,-0.0464778,-0.0472944,-0.0481114,-0.0489288,-0.0497465,-0.0505644,-0.0513827,-0.0522013,-0.0530201,-0.0538342,-0.0546436,-0.0554532,-0.0562629,-0.0570729,-0.057883,-0.0586933,-0.0595037,-0.0603144,-0.0611251,-0.061936,-0.0627471,-0.0635583,-0.0643702,-0.065183,-0.0659959,-0.066809,-0.0676221,-0.0684354,-0.0692488,-0.0700623,-0.0708759,-0.0716896,-0.0725034,-0.0733174,-0.0741314,-0.0749407,-0.0757448,-0.0765489,-0.0773531,-0.0781573,-0.0789616,-0.0797659,-0.0805703,-0.0813748,-0.0821793,-0.0829838,-0.0837884,-0.0845931,-0.0853986,-0.086205,-0.0870115,-0.0878181,-0.0886247,-0.0894313,-0.090238,-0.0910447,-0.0918514,-0.0926582,-0.093465,-0.0942719,-0.0950788,-0.0958868,-0.0966957,-0.0975047,-0.0983136,-0.0991227,-0.0999317,-0.100741,-0.10155,-0.102359,-0.103168,-0.103977,-0.104787,-0.105596,-0.106399,-0.107198,-0.107997,-0.108797,-0.109596,-0.110395,-0.111194,-0.111994,-0.112793,-0.113592,-0.114392,-0.115191,-0.11599,-0.116792,-0.117593,-0.118395,-0.119197,-0.119998,-0.1208,-0.121602,-0.122404,-0.123205,-0.124007,-0.124809,-0.125611,-0.126413,-0.127205,-0.127997,-0.128789,-0.129581,-0.130372,-0.131164,-0.131956,-0.132748,-0.13354,-0.134331,-0.135123,-0.135915,-0.136707,-0.137502,-0.138296,-0.139091,-0.139885,-0.140679,-0.141474,-0.142268,-0.143063,-0.143857,-0.144652,-0.145446,-0.146241,-0.147031,-0.147816,-0.1486,-0.149385,-0.150169,-0.150954,-0.151738,-0.152523,-0.153307,-0.154092,-0.154876,-0.155661,-0.156445,-0.157223,-0.157997,-0.158772,-0.159546,-0.160321,-0.161095,-0.16187,-0.162644,-0.163419,-0.164193,-0.164967,-0.165742,-0.166516,-0.167294,-0.168071,-0.168849,-0.169626,-0.170404,-0.171181,-0.171958,-0.172736,-0.173513,-0.174291,-0.175068,-0.175845,-0.176618,-0.177386,-0.178153,-0.17892,-0.179688,-0.180455,-0.181223,-0.18199,-0.182758,-0.183525,-0.184292,-0.18506,-0.185827,-0.186598,-0.187368,-0.188139,-0.188909,-0.18968,-0.190451,-0.191221,-0.191992,-0.192763,-0.193533,-0.194304,-0.195074,-0.19584,-0.196601,-0.197362,-0.198123,-0.198883,-0.199644,-0.200405,-0.201165,-0.201926,-0.202687,-0.203447,-0.204208,-0.204968,-0.205719,-0.20647,-0.20722,-0.207971,-0.208722,-0.209472,-0.210223,-0.210974,-0.211724,-0.212475,-0.213225,-0.213976,-0.214729,-0.215483,-0.216237,-0.216992,-0.217746,-0.2185,-0.219254,-0.220008,-0.220762,-0.221516,-0.22227,-0.223024,-0.223775,-0.224519,-0.225263,-0.226007,-0.226752,-0.227496,-0.22824,-0.228984,-0.229728,-0.230472,-0.231216,-0.23196,-0.232705,-0.233453,-0.234201,-0.234949,-0.235696,-0.236444,-0.237192,-0.23794,-0.238688,-0.239435,-0.240183,-0.240931,-0.241679,-0.242417,-0.243155,-0.243892,-0.24463,-0.245368,-0.246106,-0.246844,-0.247581,-0.248319,-0.249057,-0.249795,-0.250532,-0.251261,-0.251989,-0.252717,-0.253445,-0.254172,-0.2549,-0.255628,-0.256355,-0.257083,-0.257811,-0.258538,-0.259266,-0.259997,-0.260729,-0.26146,-0.262192,-0.262923,-0.263655,-0.264386,-0.265118,-0.265849,-0.266581,-0.267312,-0.268044,-0.268767,-0.269488,-0.27021,-0.270931,-0.271652,-0.272374,-0.273095,-0.273817,-0.274538,-0.275259,-0.275981,-0.276702,-0.277413,-0.278125,-0.278836,-0.279547,-0.280258,-0.280969,-0.28168,-0.282391,-0.283102,-0.283814,-0.284525,-0.285236,-0.285951,-0.286667,-0.287382,-0.288097,-0.288812,-0.289527,-0.290242,-0.290957,-0.291672,-0.292388,-0.293103,-0.293814,-0.294518,-0.295223,-0.295928,-0.296633,-0.297338,-0.298042,-0.298747,-0.299452,-0.300156,-0.300861,-0.301566,-0.302274,-0.302982,-0.303691,-0.3044,-0.305109,-0.305818,-0.306527,-0.307235,-0.307944,-0.308653,-0.309362,-0.31007,-0.310768,-0.311466,-0.312165,-0.312863,-0.313562,-0.31426,-0.314958,-0.315657,-0.316355,-0.317053,-0.317751,-0.318452,-0.319155,-0.319857,-0.32056,-0.321262,-0.321965,-0.322667,-0.32337,-0.324072,-0.324775,-0.325477,-0.326179,-0.32687,-0.327562,-0.328254,-0.328946,-0.329638,-0.33033,-0.331022,-0.331713,-0.332405,-0.333097,-0.333789,-0.334484,-0.33518,-0.335876,-0.336572,-0.337268,-0.337964,-0.33866,-0.339356,-0.340052,-0.340748,-0.341444,-0.342136,-0.342822,-0.343507,-0.344192,-0.344877,-0.345563,-0.346248,-0.346933,-0.347618,-0.348303,-0.348989,-0.349673,-0.350347,-0.351021,-0.351695,-0.35237,-0.353044,-0.353718,-0.354392,-0.355067,-0.355741,-0.356415,-0.357089,-0.357767,-0.358446,-0.359124,-0.359803,-0.360481,-0.36116,-0.361838,-0.362516,-0.363195,-0.363873,-0.364552,-0.365233,-0.365916,-0.366599,-0.367281,-0.367964,-0.368647,-0.369329,-0.370012,-0.370695,-0.371377,-0.37206,-0.372734,-0.373406,-0.374077,-0.374748,-0.37542,-0.376091,-0.376763,-0.377434,-0.378105,-0.378777,-0.379448,-0.380123,-0.380799,-0.381475,-0.38215,-0.382826,-0.383502,-0.384177,-0.384853,-0.385529,-0.386204,-0.38688,-0.387545,-0.388209,-0.388873,-0.389538,-0.390202,-0.390866,-0.39153,-0.392194,-0.392859,-0.393523,-0.394187,-0.394856,-0.395524,-0.396193,-0.396861,-0.397529,-0.398198,-0.398866,-0.399535,-0.400203,-0.400871,-0.401536,-0.402192,-0.402849,-0.403506,-0.404163,-0.404819,-0.405476,-0.406133,-0.406789,-0.407446,-0.408103,-0.408762,-0.409423,-0.410084,-0.410745,-0.411406,-0.412066,-0.412727,-0.413388,-0.414049,-0.41471,-0.415371,-0.416036,-0.416701,-0.417366,-0.418032,-0.418697,-0.419362,-0.420027,-0.420692,-0.421357,-0.422022,-0.422681,-0.423334,-0.423987,-0.42464,-0.425293,-0.425946,-0.426599,-0.427252,-0.427905,-0.428558,-0.429212,-0.429869,-0.430526,-0.431183,-0.431841,-0.432498,-0.433155,-0.433812,-0.43447,-0.435127,-0.435784,-0.436444,-0.437105,-0.437767,-0.438428,-0.43909,-0.439751,-0.440413,-0.441074,-0.441736,-0.442397,-0.443055,-0.443704,-0.444353,-0.445002,-0.445651,-0.4463,-0.446949,-0.447598,-0.448247,-0.448896,-0.449545,-0.450198,-0.450851,-0.451504,-0.452157,-0.452811,-0.453464,-0.454117,-0.45477,-0.455423,-0.456076,-0.456719,-0.457359,-0.457999,-0.45864,-0.45928,-0.459921,-0.460561,-0.461201,-0.461842,-0.462482,-0.463138,-0.4638,-0.464461,-0.465123,-0.465784,-0.466446,-0.467108,-0.467769,-0.468431,-0.469092,-0.469745,-0.470394,-0.471042,-0.471691,-0.47234,-0.472988,-0.473637,-0.474286,-0.474934,-0.475583,-0.476223,-0.476858,-0.477494,-0.478129,-0.478764,-0.4794,-0.480035,-0.480671,-0.481306,-0.481942,-0.48258,-0.48322,-0.483859,-0.484499,-0.485138,-0.485778,-0.486417,-0.487057,-0.487696,-0.488335,-0.488978,-0.489622,-0.490266,-0.490909,-0.491553,-0.492196,-0.49284,-0.493483,-0.494127,-0.494771,-0.495418,-0.496066,-0.496713,-0.497361,-0.498009,-0.498656,-0.499304,-0.499952,-0.500599},
		{0,-2.25399e-19,-7.02899e-13,-1.17392e-10,-1.70259e-09,-9.28578e-09,-3.09042e-08,-7.71601e-08,-1.60265e-07,-2.93403e-07,-4.90349e-07,-7.6525e-07,-1.13249e-06,-1.60659e-06,-2.20222e-06,-2.93408e-06,-3.81694e-06,-4.8656e-06,-6.0949e-06,-7.5197e-06,-9.15486e-06,-1.10153e-05,-1.31158e-05,-1.54714e-05,-1.80969e-05,-2.10072e-05,-2.42173e-05,-2.77421e-05,-3.15964e-05,-3.57953e-05,-4.03536e-05,-4.52862e-05,-5.06081e-05,-5.63342e-05,-6.24795e-05,-6.90587e-05,-7.60869e-05,-8.35791e-05,-9.155e-05,-0.000100015,-0.000108988,-0.000118485,-0.00012852,-0.000139109,-0.000150267,-0.000162007,-0.000174346,-0.000187298,-0.000200878,-0.000215101,-0.000229982,-0.000245535,-0.000261777,-0.000278721,-0.000296383,-0.000314777,-0.000333919,-0.000353823,-0.000374505,-0.000395978,-0.000418259,-0.000441362,-0.000465302,-0.000490094,-0.000515752,-0.000542293,-0.000573174,-0.000607192,-0.000641373,-0.000675708,-0.000710194,-0.000744823,-0.00077959,-0.000814491,-0.00084952,-0.000884672,-0.000919944,-0.00095533,-0.000990827,-0.00103201,-0.00107888,-0.00112591,-0.00117308,-0.00122039,-0.00126784,-0.00131543,-0.00136314,-0.00141097,-0.00145893,-0.001507,-0.00155518,-0.00160347,-0.0016573,-0.00171732,-0.00177747,-0.00183776,-0.00189818,-0.00195873,-0.0020194,-0.00208019,-0.0021411,-0.00220213,-0.00226326,-0.0023245,-0.00238584,-0.00245281,-0.00252616,-0.00259964,-0.00267324,-0.00274697,-0.00282081,-0.00289476,-0.00296883,-0.00304301,-0.00311729,-0.00319167,-0.00326616,-0.00334075,-0.00342721,-0.00352594,-0.00362481,-0.00372381,-0.00382296,-0.00392223,-0.00402163,-0.00412116,-0.00422081,-0.00432058,-0.00442047,-0.00452047,-0.00462059,-0.00472757,-0.00484023,-0.00495302,-0.00506592,-0.00517895,-0.0052921,-0.00540537,-0.00551874,-0.00563223,-0.00574583,-0.00585953,-0.00597334,-0.00608725,-0.00621706,-0.00635594,-0.00649496,-0.0066341,-0.00677336,-0.00691275,-0.00705226,-0.00719189,-0.00733164,-0.0074715,-0.00761147,-0.00775155,-0.00789174,-0.00804183,-0.00819503,-0.00834834,-0.00850177,-0.00865531,-0.00880896,-0.00896272,-0.00911659,-0.00927056,-0.00942463,-0.00957881,-0.00973309,-0.00988746,-0.0100657,-0.0102458,-0.010426,-0.0106063,-0.0107867,-0.0109672,-0.0111479,-0.0113286,-0.0115095,-0.0116905,-0.0118715,-0.0120527,-0.0122359,-0.0124305,-0.0126252,-0.01282,-0.013015,-0.01321,-0.0134052,-0.0136004,-0.0137957,-0.0139912,-0.0141867,-0.0143823,-0.0145781,-0.0147844,-0.0150065,-0.0152286,-0.0154509,-0.0156733,-0.0158959,-0.0161185,-0.0163412,-0.016564,-0.0167869,-0.0170099,-0.017233,-0.0174562,-0.0176892,-0.0179263,-0.0181635,-0.0184008,-0.0186381,-0.0188756,-0.0191132,-0.0193509,-0.0195886,-0.0198265,-0.0200644,-0.0203024,-0.0205414,-0.0207935,-0.0210457,-0.021298,-0.0215504,-0.0218029,-0.0220555,-0.0223081,-0.0225608,-0.0228137,-0.0230666,-0.0233195,-0.0235726,-0.0238385,-0.024119,-0.0243994,-0.02468,-0.0249607,-0.0252415,-0.0255223,-0.0258033,-0.0260843,-0.0263654,-0.0266466,-0.0269279,-0.0272093,-0.0275042,-0.0278001,-0.0280961,-0.0283922,-0.0286884,-0.0289847,-0.029281,-0.0295774,-0.0298739,-0.0301705,-0.0304672,-0.0307639,-0.0310674,-0.0313789,-0.0316905,-0.0320022,-0.032314,-0.0326258,-0.0329377,-0.0332497,-0.0335618,-0.0338739,-0.0341861,-0.0344984,-0.0348113,-0.0351386,-0.0354659,-0.0357933,-0.0361208,-0.0364484,-0.036776,-0.0371038,-0.0374315,-0.0377594,-0.0380873,-0.0384153,-0.0387434,-0.0390817,-0.039425,-0.0397683,-0.0401117,-0.0404552,-0.0407988,-0.0411424,-0.0414861,-0.0418299,-0.0421737,-0.0425176,-0.0428615,-0.0432114,-0.0435707,-0.0439302,-0.0442897,-0.0446493,-0.045009,-0.0453687,-0.0457285,-0.0460883,-0.0464482,-0.0468082,-0.0471683,-0.0475306,-0.0479063,-0.0482821,-0.0486579,-0.0490338,-0.0494097,-0.0497857,-0.0501618,-0.050538,-0.0509142,-0.0512904,-0.0516667,-0.0520431,-0.0524349,-0.0528272,-0.0532195,-0.0536119,-0.0540044,-0.0543969,-0.0547894,-0.0551821,-0.0555748,-0.0559675,-0.0563603,-0.0567532,-0.057148,-0.0575431,-0.0579383,-0.0583335,-0.0587288,-0.0591242,-0.0595196,-0.059915,-0.0603105,-0.0607061,-0.0611017,-0.0614974,-0.0619066,-0.0623186,-0.0627306,-0.0631426,-0.0635547,-0.0639668,-0.064379,-0.0647913,-0.0652036,-0.0656159,-0.0660283,-0.0664408,-0.0668554,-0.0672703,-0.0676852,-0.0681002,-0.0685153,-0.0689304,-0.0693455,-0.0697607,-0.070176,-0.0705913,-0.0710066,-0.071422,-0.0718398,-0.0722576,-0.0726756,-0.0730936,-0.0735116,-0.0739296,-0.0743477,-0.0747659,-0.0751841,-0.0756023,-0.0760206,-0.0764414,-0.0768765,-0.0773116,-0.0777467,-0.078182,-0.0786172,-0.0790525,-0.0794878,-0.0799232,-0.0803587,-0.0807941,-0.0812296,-0.0816662,-0.0821044,-0.0825426,-0.0829809,-0.0834192,-0.0838576,-0.084296,-0.0847344,-0.0851729,-0.0856114,-0.08605,-0.0864886,-0.0869393,-0.087395,-0.0878509,-0.0883067,-0.0887626,-0.0892186,-0.0896746,-0.0901306,-0.0905867,-0.0910428,-0.0914989,-0.091954,-0.0923985,-0.092843,-0.0932875,-0.093732,-0.0941766,-0.0946212,-0.0950659,-0.0955106,-0.0959553,-0.0964,-0.0968448,-0.0972992,-0.0977615,-0.0982238,-0.0986861,-0.0991485,-0.0996109,-0.100073,-0.100536,-0.100998,-0.101461,-0.101924,-0.102385,-0.102836,-0.103287,-0.103738,-0.104188,-0.104639,-0.10509,-0.105541,-0.105992,-0.106443,-0.106894,-0.107345,-0.107808,-0.108277,-0.108746,-0.109215,-0.109684,-0.110153,-0.110622,-0.111091,-0.11156,-0.112029,-0.112498,-0.112963,-0.11342,-0.113877,-0.114335,-0.114792,-0.115249,-0.115706,-0.116164,-0.116621,-0.117079,-0.117536,-0.117995,-0.118471,-0.118946,-0.119422,-0.119897,-0.120373,-0.120848,-0.121324,-0.1218,-0.122275,-0.122751,-0.123227,-0.123691,-0.124155,-0.124619,-0.125082,-0.125546,-0.12601,-0.126473,-0.126937,-0.127401,-0.127865,-0.128329,-0.128795,-0.129262,-0.129729,-0.130196,-0.130663,-0.13113,-0.131597,-0.132064,-0.132531,-0.132998,-0.133465,-0.133934,-0.134404,-0.134874,-0.135345,-0.135815,-0.136285,-0.136755,-0.137226,-0.137696,-0.138166,-0.138636,-0.139097,-0.139555,-0.140012,-0.14047,-0.140928,-0.141386,-0.141844,-0.142302,-0.14276,-0.143218,-0.143676,-0.144137,-0.144598,-0.145059,-0.14552,-0.145982,-0.146443,-0.146904,-0.147365,-0.147826,-0.148287,-0.148749,-0.149213,-0.149677,-0.150142,-0.150606,-0.15107,-0.151535,-0.151999,-0.152463,-0.152928,-0.153392,-0.153858,-0.154325,-0.154793,-0.15526,-0.155728,-0.156195,-0.156663,-0.15713,-0.157598,-0.158065,-0.158533,-0.158992,-0.159447,-0.159901,-0.160356,-0.160811,-0.161266,-0.16172,-0.162175,-0.16263,-0.163085,-0.163539,-0.163981,-0.164422,-0.164864,-0.165306,-0.165748,-0.166189,-0.166631,-0.167073,-0.167515,-0.167957,-0.168408,-0.168869,-0.16933,-0.169791,-0.170252,-0.170713,-0.171174,-0.171635,-0.172096,-0.172557,-0.173018,-0.173466,-0.173913,-0.174361,-0.174809,-0.175257,-0.175705,-0.176152,-0.1766,-0.177048,-0.177496,-0.177935,-0.178369,-0.178804,-0.179238,-0.179672,-0.180107,-0.180541,-0.180976,-0.18141,-0.181845,-0.18228,-0.182717,-0.183154,-0.183592,-0.184029,-0.184466,-0.184904,-0.185341,-0.185778,-0.186216,-0.186652,-0.187076,-0.187499,-0.187923,-0.188346,-0.18877,-0.189194,-0.189617,-0.190041,-0.190464,-0.190888,-0.191314,-0.19174,-0.192167,-0.192593,-0.193019,-0.193446,-0.193872,-0.194299,-0.194725,-0.195152,-0.19558,-0.196009,-0.196438,-0.196868,-0.197297,-0.197726,-0.198155,-0.198585,-0.199014,-0.199443,-0.199852,-0.200249,-0.200647,-0.201045,-0.201443,-0.201841,-0.202239,-0.202637,-0.203035,-0.203433,-0.203845,-0.204262,-0.20468,-0.205098,-0.205516,-0.205933,-0.206351,-0.206769,-0.207187,-0.207605,-0.207999,-0.208385,-0.208771,-0.209157,-0.209543,-0.209929,-0.210315,-0.210701,-0.211086,-0.211472,-0.211861,-0.212249,-0.212637,-0.213026,-0.213414,-0.213803,-0.214191,-0.21458,-0.214968,-0.215357,-0.215748,-0.216139,-0.21653,-0.216921,-0.217311,-0.217702,-0.218093,-0.218484,-0.218875},
		{0,1.12699e-19,3.51449e-13,5.86958e-11,8.51296e-10,4.64289e-09,1.54521e-08,3.858e-08,8.01325e-08,1.46701e-07,2.45175e-07,3.82625e-07,5.66243e-07,8.03297e-07,1.10111e-06,1.46704e-06,1.90847e-06,2.4328e-06,3.04745e-06,3.75985e-06,4.57743e-06,5.50763e-06,6.5579e-06,7.73568e-06,9.04843e-06,1.05036e-05,1.21087e-05,1.3871e-05,1.57982e-05,1.78977e-05,2.01768e-05,2.26431e-05,2.53041e-05,2.81671e-05,3.12397e-05,3.45294e-05,3.80435e-05,4.17895e-05,4.5775e-05,5.00073e-05,5.4494e-05,5.92425e-05,6.42602e-05,6.95546e-05,7.51333e-05,8.10035e-05,8.71729e-05,9.36489e-05,0.000100439,0.00010755,0.000114991,0.000122768,0.000130888,0.000139361,0.000148191,0.000157389,0.000166959,0.000176912,0.000187252,0.000197989,0.00020913,0.000220681,0.000232651,0.000245047,0.000257876,0.000271146,0.000283654,0.000295313,0.000307017,0.000318765,0.000330556,0.000342386,0.000354256,0.000366163,0.000378105,0.000390083,0.000402094,0.000414137,0.000426211,0.000443874,0.000467138,0.000490482,0.000513902,0.000537397,0.000560964,0.000584601,0.000608305,0.000632074,0.000655906,0.000679799,0.000703752,0.000727762,0.000751849,0.000776012,0.000800228,0.000824495,0.000848811,0.000873176,0.000897587,0.000922044,0.000946544,0.000971088,0.000995673,0.0010203,0.00104496,0.00107515,0.00111162,0.00114816,0.00118477,0.00122143,0.00125816,0.00129495,0.0013318,0.00136871,0.00140567,0.00144269,0.00147976,0.00151688,0.0015541,0.00159141,0.00162877,0.00166617,0.00170362,0.00174112,0.00177866,0.00181625,0.00185387,0.00189154,0.00192925,0.001967,0.00200478,0.00204927,0.0020993,0.00214938,0.00219952,0.00224971,0.00229996,0.00235025,0.0024006,0.002451,0.00250145,0.00255194,0.00260248,0.00265307,0.00271159,0.00277464,0.00283775,0.00290091,0.00296414,0.00302742,0.00309075,0.00315415,0.00321759,0.00328109,0.00334464,0.00340824,0.00347189,0.00354521,0.00362152,0.00369789,0.00377433,0.00385082,0.00392738,0.00400399,0.00408065,0.00415738,0.00423415,0.00431098,0.00438787,0.0044648,0.00454203,0.00461931,0.00469665,0.00477404,0.00485147,0.00492895,0.00500648,0.00508405,0.00516167,0.00523933,0.00531704,0.00539478,0.00547444,0.00556515,0.00565591,0.00574673,0.0058376,0.00592851,0.00601948,0.00611049,0.00620155,0.00629265,0.0063838,0.006475,0.00656624,0.00666276,0.00676716,0.0068716,0.0069761,0.00708065,0.00718525,0.00728989,0.00739459,0.00749933,0.00760412,0.00770896,0.00781384,0.00791877,0.00802403,0.00812946,0.00823493,0.00834044,0.00844599,0.00855159,0.00865723,0.00876291,0.00886862,0.00897438,0.00908018,0.00918601,0.00929352,0.00942575,0.00955803,0.00969036,0.00982274,0.00995517,0.0100877,0.0102202,0.0103528,0.0104854,0.0106181,0.0107508,0.0108836,0.0110166,0.0111501,0.0112835,0.0114171,0.0115506,0.0116842,0.0118179,0.0119516,0.0120853,0.0122191,0.0123529,0.0124867,0.0126206,0.0127551,0.0128897,0.0130243,0.0131589,0.0132936,0.0134283,0.0135631,0.0136979,0.0138327,0.0139675,0.0141024,0.0142374,0.0143847,0.0145468,0.014709,0.0148712,0.0150334,0.0151957,0.0153581,0.0155204,0.0156829,0.0158453,0.0160078,0.0161704,0.016333,0.0164964,0.0166598,0.0168233,0.0169868,0.0171504,0.017314,0.0174776,0.0176413,0.017805,0.0179688,0.0181326,0.0182964,0.0184699,0.018648,0.0188262,0.0190044,0.0191827,0.019361,0.0195393,0.0197177,0.0198961,0.0200746,0.0202531,0.0204316,0.0206105,0.02079,0.0209695,0.0211491,0.0213287,0.0215084,0.021688,0.0218677,0.0220475,0.0222272,0.0224071,0.0225869,0.0227689,0.0229634,0.0231579,0.0233524,0.023547,0.0237417,0.0239363,0.024131,0.0243257,0.0245205,0.0247153,0.0249101,0.025105,0.0253009,0.0254969,0.0256929,0.0258889,0.0260849,0.026281,0.0264772,0.0266733,0.0268695,0.0270657,0.0272619,0.0274582,0.0276676,0.0278788,0.0280901,0.0283014,0.0285127,0.0287241,0.0289355,0.029147,0.0293584,0.0295699,0.0297814,0.029993,0.0302056,0.0304184,0.0306312,0.0308441,0.0310569,0.0312699,0.0314828,0.0316958,0.0319088,0.0321218,0.0323348,0.0325479,0.0327743,0.0330027,0.0332311,0.0334596,0.0336881,0.0339166,0.0341451,0.0343737,0.0346023,0.0348309,0.0350595,0.0352882,0.0355182,0.0357483,0.0359784,0.0362085,0.0364386,0.0366688,0.036899,0.0371292,0.0373594,0.0375897,0.03782,0.0380505,0.0382822,0.0385139,0.0387457,0.0389774,0.0392092,0.0394411,0.0396729,0.0399048,0.0401367,0.0403686,0.0406005,0.0408386,0.0410863,0.0413341,0.0415819,0.0418297,0.0420775,0.0423253,0.0425732,0.0428211,0.043069,0.043317,0.0435649,0.043814,0.0440635,0.044313,0.0445626,0.0448122,0.0450618,0.0453114,0.0455611,0.0458107,0.0460604,0.0463102,0.04656,0.0468113,0.0470626,0.047314,0.0475653,0.0478167,0.0480681,0.0483195,0.048571,0.0488224,0.0490739,0.0493254,0.0495778,0.0498309,0.050084,0.0503372,0.0505903,0.0508435,0.0510967,0.0513499,0.0516032,0.0518564,0.0521097,0.0523642,0.0526339,0.0529036,0.0531733,0.053443,0.0537128,0.0539826,0.0542523,0.0545221,0.054792,0.0550618,0.0553317,0.0555926,0.0558494,0.0561061,0.0563629,0.0566197,0.0568765,0.0571333,0.0573901,0.057647,0.0579038,0.0581607,0.0584235,0.058697,0.0589706,0.0592441,0.0595177,0.0597913,0.0600649,0.0603386,0.0606122,0.0608859,0.0611596,0.0614335,0.0617089,0.0619844,0.0622599,0.0625354,0.062811,0.0630865,0.0633621,0.0636377,0.0639133,0.0641889,0.0644646,0.0647419,0.0650193,0.0652968,0.0655742,0.0658517,0.0661292,0.0664067,0.0666843,0.0669618,0.0672394,0.067517,0.0677836,0.0680477,0.0683118,0.0685759,0.0688401,0.0691042,0.0693684,0.0696325,0.0698967,0.0701609,0.0704251,0.0707026,0.070984,0.0712654,0.0715468,0.0718282,0.0721096,0.072391,0.0726725,0.072954,0.0732354,0.0735169,0.0737999,0.0740833,0.0743666,0.07465,0.0749334,0.0752168,0.0755002,0.0757837,0.0760671,0.0763506,0.076634,0.0769192,0.0772046,0.0774899,0.0777753,0.0780607,0.0783461,0.0786315,0.0789169,0.0792023,0.0794878,0.0797734,0.0800607,0.0803481,0.0806354,0.0809228,0.0812102,0.0814976,0.081785,0.0820724,0.0823598,0.0826473,0.0829298,0.0832033,0.0834768,0.0837503,0.0840238,0.0842973,0.0845708,0.0848444,0.0851179,0.0853915,0.085665,0.0859505,0.0862419,0.0865332,0.0868246,0.087116,0.0874074,0.0876988,0.0879902,0.0882817,0.0885731,0.0888647,0.0891581,0.0894514,0.0897448,0.0900382,0.0903317,0.0906251,0.0909185,0.091212,0.0915054,0.0917989,0.0920849,0.0923641,0.0926433,0.0929225,0.0932017,0.093481,0.0937602,0.0940395,0.0943187,0.094598,0.0948783,0.0951757,0.0954731,0.0957706,0.0960681,0.0963655,0.096663,0.0969605,0.097258,0.0975555,0.0978531,0.0981411,0.0984242,0.0987072,0.0989903,0.0992734,0.0995564,0.0998395,0.100123,0.100406,0.100689,0.100978,0.101279,0.101581,0.101882,0.102184,0.102486,0.102787,0.103089,0.10339,0.103692,0.103993,0.10428,0.104567,0.104853,0.10514,0.105427,0.105714,0.106001,0.106288,0.106575,0.106862,0.107151,0.10744,0.107729,0.108017,0.108306,0.108595,0.108884,0.109173,0.109462,0.109751,0.110041,0.110332,0.110623,0.110914,0.111204,0.111495,0.111786,0.112077,0.112368,0.112659,0.112951,0.113243,0.113536,0.113829,0.114122,0.114415,0.114707,0.115,0.115293,0.115586,0.11588,0.116175,0.116469,0.116764,0.117059,0.117353,0.117648,0.117943,0.118238,0.118532,0.118841,0.119155,0.119469,0.119783,0.120097,0.120411,0.120725,0.121039,0.121353,0.121667,0.121968,0.122266,0.122565,0.122864,0.123162,0.123461,0.12376,0.124058,0.124357,0.124656,0.124956,0.125257,0.125558,0.125858,0.126159,0.126459,0.12676,0.127061,0.127361}};
	double data_ddeltadq[3][701]={{0,-1.90414e-13,-1.31509e-07,-8.20203e-06,-5.47108e-05,-0.000154941,-0.00029183,-0.000440948,-0.000585309,-0.000716492,-0.000831718,-0.000931159,-0.00101628,-0.00108894,-0.0011512,-0.0012044,-0.00125013,-0.0012896,-0.00132383,-0.00135365,-0.00137975,-0.0014027,-0.00142296,-0.00144092,-0.00145692,-0.00147121,-0.00148402,-0.00151099,-0.00152209,-0.00153215,-0.0015413,-0.00154964,-0.00155726,-0.00156425,-0.00157066,-0.00157657,-0.00158202,-0.00158706,-0.00159172,-0.00159605,-0.00156475,-0.0015678,-0.00157064,-0.00157329,-0.00157576,-0.00157808,-0.00158025,-0.00158228,-0.00158419,-0.00158599,-0.00158768,-0.00158928,-0.00159079,-0.00161357,-0.00161516,-0.00161666,-0.00161809,-0.00161944,-0.00162073,-0.00162195,-0.00162312,-0.00162423,-0.00162529,-0.0016263,-0.00162727,-0.00162819,-0.00163089,-0.00163175,-0.00163257,-0.00163336,-0.00163411,-0.00163484,-0.00163554,-0.00163621,-0.00163685,-0.00163747,-0.00163807,-0.00163865,-0.0016392,-0.00161986,-0.00162028,-0.00162068,-0.00162107,-0.00162145,-0.00162181,-0.00162216,-0.0016225,-0.00162283,-0.00162314,-0.00162345,-0.00162375,-0.00162404,-0.00162697,-0.00162725,-0.00162752,-0.00162779,-0.00162805,-0.0016283,-0.00162854,-0.00162878,-0.00162901,-0.00162923,-0.00162945,-0.00162967,-0.00162987,-0.00160983,-0.00160997,-0.0016101,-0.00161024,-0.00161037,-0.00161049,-0.00161062,-0.00161074,-0.00161086,-0.00161097,-0.00161109,-0.0016112,-0.00161131,-0.00161486,-0.00161497,-0.00161508,-0.00161519,-0.00161529,-0.00161539,-0.0016155,-0.00161559,-0.00161569,-0.00161579,-0.00161588,-0.00161597,-0.00161606,-0.00162001,-0.0016201,-0.00162019,-0.00162028,-0.00162037,-0.00162046,-0.00162055,-0.00162063,-0.00162072,-0.0016208,-0.00162088,-0.00162096,-0.00162104,-0.0016009,-0.00160095,-0.00160099,-0.00160104,-0.00160108,-0.00160112,-0.00160117,-0.00160121,-0.00160125,-0.00160129,-0.00160133,-0.00160138,-0.00160142,-0.00160604,-0.00160608,-0.00160613,-0.00160617,-0.00160622,-0.00160626,-0.0016063,-0.00160635,-0.00160639,-0.00160643,-0.00160647,-0.00160651,-0.00160656,-0.00158651,-0.00158653,-0.00158655,-0.00158657,-0.00158659,-0.00158661,-0.00158662,-0.00158664,-0.00158666,-0.00158668,-0.0015867,-0.00158672,-0.00159198,-0.00159201,-0.00159203,-0.00159205,-0.00159208,-0.0015921,-0.00159212,-0.00159215,-0.00159217,-0.00159219,-0.00159222,-0.00159224,-0.00159226,-0.00157237,-0.00157238,-0.00157238,-0.00157239,-0.0015724,-0.0015724,-0.00157241,-0.00157242,-0.00157243,-0.00157243,-0.00157244,-0.00157245,-0.00157246,-0.00155257,-0.00155256,-0.00155256,-0.00155255,-0.00155255,-0.00155254,-0.00155254,-0.00155253,-0.00155253,-0.00155253,-0.00155252,-0.00155252,-0.00155861,-0.00155861,-0.00155861,-0.00155861,-0.00155862,-0.00155862,-0.00155862,-0.00155862,-0.00155863,-0.00155863,-0.00155863,-0.00155864,-0.00155864,-0.00153887,-0.00153886,-0.00153886,-0.00153885,-0.00153884,-0.00153884,-0.00153883,-0.00153883,-0.00153882,-0.00153881,-0.00153881,-0.00153881,-0.0015388,-0.00154541,-0.00154541,-0.00154541,-0.00154541,-0.00154541,-0.00154541,-0.00154541,-0.00154541,-0.00154542,-0.00154542,-0.00154542,-0.00154542,-0.00152572,-0.00152572,-0.00152571,-0.0015257,-0.0015257,-0.00152569,-0.00152569,-0.00152568,-0.00152567,-0.00152567,-0.00152566,-0.00152566,-0.00150591,-0.0015059,-0.00150589,-0.00150588,-0.00150586,-0.00150585,-0.00150584,-0.00150583,-0.00150582,-0.00150581,-0.0015058,-0.00150579,-0.00150578,-0.00151298,-0.00151298,-0.00151297,-0.00151297,-0.00151296,-0.00151296,-0.00151295,-0.00151295,-0.00151294,-0.00151294,-0.00151293,-0.00151293,-0.00149318,-0.00149316,-0.00149315,-0.00149314,-0.00149313,-0.00149312,-0.00149311,-0.0014931,-0.0014931,-0.00149309,-0.00149308,-0.00149307,-0.00150065,-0.00150064,-0.00150064,-0.00150063,-0.00150063,-0.00150062,-0.00150062,-0.00150061,-0.00150061,-0.00150061,-0.0015006,-0.0015006,-0.0015006,-0.00148078,-0.00148077,-0.00148076,-0.00148075,-0.00148074,-0.00148073,-0.00148073,-0.00148072,-0.00148071,-0.0014807,-0.0014807,-0.00148069,-0.00146074,-0.00146073,-0.00146071,-0.0014607,-0.00146069,-0.00146068,-0.00146067,-0.00146066,-0.00146065,-0.00146064,-0.00146063,-0.00146062,-0.00146857,-0.00146856,-0.00146855,-0.00146855,-0.00146854,-0.00146853,-0.00146853,-0.00146852,-0.00146851,-0.00146851,-0.0014685,-0.0014685,-0.00144839,-0.00144838,-0.00144837,-0.00144836,-0.00144835,-0.00144835,-0.00144834,-0.00144833,-0.00144832,-0.00144831,-0.0014483,-0.00144829,-0.00142801,-0.00142799,-0.00142798,-0.00142797,-0.00142796,-0.00142794,-0.00142793,-0.00142792,-0.00142791,-0.0014279,-0.00142789,-0.00143609,-0.00143608,-0.00143607,-0.00143606,-0.00143605,-0.00143604,-0.00143604,-0.00143603,-0.00143602,-0.00143601,-0.00143601,-0.001436,-0.00141549,-0.00141547,-0.00141546,-0.00141545,-0.00141544,-0.00141543,-0.00141542,-0.00141541,-0.0014154,-0.00141539,-0.00141538,-0.00141537,-0.00142374,-0.00142373,-0.00142372,-0.00142372,-0.00142371,-0.0014237,-0.0014237,-0.00142369,-0.00142368,-0.00142368,-0.00142367,-0.00140287,-0.00140287,-0.00140286,-0.00140285,-0.00140284,-0.00140283,-0.00140282,-0.00140281,-0.0014028,-0.00140279,-0.00140278,-0.00140278,-0.00141126,-0.00141126,-0.00141125,-0.00141125,-0.00141124,-0.00141123,-0.00141123,-0.00141122,-0.00141122,-0.00141121,-0.00141121,-0.00139008,-0.00139007,-0.00139006,-0.00139006,-0.00139005,-0.00139004,-0.00139003,-0.00139002,-0.00139002,-0.00139001,-0.00139,-0.00138999,-0.00139856,-0.00139856,-0.00139855,-0.00139855,-0.00139854,-0.00139854,-0.00139853,-0.00139853,-0.00139853,-0.00139852,-0.00139852,-0.00137701,-0.001377,-0.001377,-0.00137699,-0.00137698,-0.00137698,-0.00137697,-0.00137696,-0.00137696,-0.00137695,-0.00137694,-0.00135514,-0.00135513,-0.00135512,-0.00135511,-0.0013551,-0.00135509,-0.00135508,-0.00135507,-0.00135506,-0.00135505,-0.00135505,-0.00135504,-0.00136356,-0.00136356,-0.00136355,-0.00136354,-0.00136354,-0.00136353,-0.00136352,-0.00136352,-0.00136351,-0.00136351,-0.0013635,-0.00137213,-0.00137212,-0.00137212,-0.00137212,-0.00137211,-0.00137211,-0.00137211,-0.0013721,-0.0013721,-0.0013721,-0.00137209,-0.00134972,-0.00134971,-0.00134971,-0.0013497,-0.00134969,-0.00134969,-0.00134968,-0.00134968,-0.00134967,-0.00134967,-0.00134966,-0.00135827,-0.00135827,-0.00135826,-0.00135826,-0.00135826,-0.00135825,-0.00135825,-0.00135825,-0.00135824,-0.00135824,-0.00135824,-0.00133538,-0.00133537,-0.00133536,-0.00133536,-0.00133535,-0.00133535,-0.00133534,-0.00133534,-0.00133533,-0.00133533,-0.00134389,-0.00134389,-0.00134388,-0.00134388,-0.00134388,-0.00134387,-0.00134387,-0.00134387,-0.00134386,-0.00134386,-0.00134386,-0.00132048,-0.00132047,-0.00132047,-0.00132046,-0.00132046,-0.00132045,-0.00132045,-0.00132044,-0.00132044,-0.00132043,-0.00132043,-0.00132892,-0.00132892,-0.00132891,-0.00132891,-0.00132891,-0.0013289,-0.0013289,-0.0013289,-0.00132889,-0.00132889,-0.00133745,-0.00133745,-0.00133745,-0.00133745,-0.00133744,-0.00133744,-0.00133744,-0.00133744,-0.00133744,-0.00133744,-0.00133744,-0.00131334,-0.00131333,-0.00131333,-0.00131333,-0.00131332,-0.00131332,-0.00131332,-0.00131331,-0.00131331,-0.00131331,-0.00132176,-0.00132176,-0.00132176,-0.00132176,-0.00132176,-0.00132176,-0.00132175,-0.00132175,-0.00132175,-0.00132175,-0.00132175,-0.00133026,-0.00133026,-0.00133026,-0.00133026,-0.00133026,-0.00133026,-0.00133026,-0.00133026,-0.00133026,-0.00133026,-0.00130539,-0.00130539,-0.00130539,-0.00130538,-0.00130538,-0.00130538,-0.00130538,-0.00130538,-0.00130537,-0.00130537,-0.00131375,-0.00131375,-0.00131375,-0.00131375,-0.00131375,-0.00131375,-0.00131375,-0.00131375,-0.00131375,-0.00131375,-0.00131375,-0.00128827,-0.00128827,-0.00128826,-0.00128826,-0.00128826,-0.00128826,-0.00128825,-0.00128825,-0.00128825,-0.00128825,-0.00133063,-0.00133063,-0.00133063,-0.00133064,-0.00133064,-0.00133064,-0.00133064,-0.00133065,-0.00133065,-0.00133065,-0.00130477,-0.00130477,-0.00130477,-0.00130477,-0.00130477,-0.00130477,-0.00130477,-0.00130477,-0.00130477,-0.00130477,-0.00127845,-0.00127845,-0.00127845,-0.00127845,-0.00127845,-0.00127844,-0.00127844,-0.00127844,-0.00127844,-0.00127844,-0.00128655,-0.00128655,-0.00128655,-0.00128655,-0.00128655,-0.00128655,-0.00128654,-0.00128654,-0.00128654,-0.00128654,-0.00129468,-0.00129468,-0.00129468,-0.00129468,-0.00129468,-0.00129468,-0.00129469,-0.00129469,-0.00129469,-0.00130285,-0.00130285,-0.00130285,-0.00130285,-0.00130286,-0.00130286,-0.00130286,-0.00130286,-0.00130287,-0.00130287},
		{0,-1.34109e-17,-1.06391e-11,-8.18894e-10,-7.02839e-09,-2.60219e-08,-6.40198e-08,-1.25098e-07,-2.11735e-07,-3.25428e-07,-4.67099e-07,-6.37333e-07,-8.36509e-07,-1.06488e-06,-1.32262e-06,-1.60986e-06,-1.92668e-06,-2.27314e-06,-2.6493e-06,-3.05519e-06,-3.49083e-06,-3.95625e-06,-4.45146e-06,-4.97649e-06,-5.53133e-06,-6.116e-06,-6.7305e-06,-7.37484e-06,-8.04902e-06,-8.75305e-06,-9.48693e-06,-1.02507e-05,-1.10443e-05,-1.18677e-05,-1.2721e-05,-1.36042e-05,-1.45172e-05,-1.54601e-05,-1.64328e-05,-1.74354e-05,-1.84679e-05,-1.95302e-05,-2.06224e-05,-2.17445e-05,-2.28964e-05,-2.40782e-05,-2.52899e-05,-2.65314e-05,-2.78028e-05,-2.9104e-05,-3.04351e-05,-3.17961e-05,-3.31869e-05,-3.46076e-05,-3.60582e-05,-3.75386e-05,-3.90489e-05,-4.0589e-05,-4.2159e-05,-4.37589e-05,-4.53886e-05,-4.70482e-05,-4.87377e-05,-5.0457e-05,-5.22062e-05,-5.39852e-05,-6.78832e-05,-6.82146e-05,-6.8533e-05,-6.88389e-05,-6.91329e-05,-6.94158e-05,-6.9688e-05,-6.99501e-05,-7.02026e-05,-7.04459e-05,-7.06805e-05,-7.09068e-05,-7.11251e-05,-9.36169e-05,-9.39264e-05,-9.42254e-05,-9.45143e-05,-9.47937e-05,-9.5064e-05,-9.53255e-05,-9.55786e-05,-9.58236e-05,-9.6061e-05,-9.6291e-05,-9.65139e-05,-9.67301e-05,-0.000119928,-0.000120212,-0.000120488,-0.000120755,-0.000121015,-0.000121267,-0.000121512,-0.000121751,-0.000121982,-0.000122207,-0.000122426,-0.000122639,-0.000122847,-0.000146629,-0.000146888,-0.000147141,-0.000147388,-0.000147628,-0.000147862,-0.00014809,-0.000148312,-0.000148529,-0.000148741,-0.000148947,-0.000149148,-0.000149345,-0.000197387,-0.000197676,-0.000197959,-0.000198234,-0.000198503,-0.000198767,-0.000199024,-0.000199275,-0.000199521,-0.000199761,-0.000199995,-0.000200225,-0.00020045,-0.0002253,-0.000225558,-0.00022581,-0.000226057,-0.000226298,-0.000226535,-0.000226766,-0.000226993,-0.000227215,-0.000227433,-0.000227646,-0.000227855,-0.000228059,-0.000277773,-0.000278041,-0.000278303,-0.00027856,-0.000278812,-0.00027906,-0.000279302,-0.000279541,-0.000279774,-0.000280003,-0.000280228,-0.000280449,-0.000280666,-0.000306457,-0.000306696,-0.000306932,-0.000307163,-0.00030739,-0.000307613,-0.000307832,-0.000308047,-0.000308258,-0.000308466,-0.00030867,-0.000308871,-0.000309069,-0.00036025,-0.000360493,-0.000360732,-0.000360967,-0.000361198,-0.000361425,-0.000361648,-0.000361868,-0.000362085,-0.000362298,-0.000362507,-0.000362713,-0.000389393,-0.000389617,-0.000389837,-0.000390053,-0.000390266,-0.000390476,-0.000390683,-0.000390886,-0.000391087,-0.000391284,-0.000391479,-0.00039167,-0.000391859,-0.000444391,-0.000444615,-0.000444836,-0.000445053,-0.000445268,-0.000445479,-0.000445687,-0.000445893,-0.000446095,-0.000446295,-0.000446491,-0.000446685,-0.000446877,-0.000474484,-0.000474689,-0.00047489,-0.00047509,-0.000475286,-0.00047548,-0.000475671,-0.00047586,-0.000476046,-0.00047623,-0.000476411,-0.000476589,-0.000504594,-0.000504786,-0.000504975,-0.000505161,-0.000505345,-0.000505526,-0.000505706,-0.000505883,-0.000506057,-0.00050623,-0.0005064,-0.000506568,-0.000506734,-0.00056125,-0.000561442,-0.000561632,-0.000561819,-0.000562004,-0.000562186,-0.000562367,-0.000562545,-0.000562721,-0.000562895,-0.000563067,-0.000563237,-0.000563405,-0.000592387,-0.000592565,-0.000592741,-0.000592915,-0.000593087,-0.000593257,-0.000593425,-0.000593591,-0.000593755,-0.000593918,-0.000594078,-0.000594237,-0.000623643,-0.000623812,-0.000623978,-0.000624143,-0.000624306,-0.000624467,-0.000624626,-0.000624783,-0.000624939,-0.000625093,-0.000625246,-0.000625397,-0.000655235,-0.000655395,-0.000655553,-0.00065571,-0.000655865,-0.000656018,-0.000656169,-0.000656319,-0.000656468,-0.000656614,-0.00065676,-0.000656904,-0.000657046,-0.000687335,-0.000687486,-0.000687635,-0.000687783,-0.000687929,-0.000688074,-0.000688218,-0.00068836,-0.0006885,-0.000688639,-0.000688777,-0.000688913,-0.00071965,-0.000719794,-0.000719937,-0.000720079,-0.000720219,-0.000720358,-0.000720496,-0.000720632,-0.000720767,-0.0007209,-0.000721033,-0.000721164,-0.000752355,-0.000752494,-0.000752631,-0.000752767,-0.000752902,-0.000753036,-0.000753168,-0.000753299,-0.000753429,-0.000753558,-0.000753686,-0.000753812,-0.000753938,-0.000785599,-0.000785731,-0.000785863,-0.000785993,-0.000786122,-0.000786249,-0.000786376,-0.000786502,-0.000786626,-0.000786749,-0.000786872,-0.000786993,-0.000791428,-0.000791548,-0.000791667,-0.000791785,-0.000791903,-0.000792019,-0.000792134,-0.000792249,-0.000792362,-0.000792474,-0.000792586,-0.000792696,-0.000825134,-0.000825252,-0.000825368,-0.000825483,-0.000825597,-0.00082571,-0.000825823,-0.000825934,-0.000826045,-0.000826154,-0.000826263,-0.000826371,-0.000831187,-0.000831294,-0.000831401,-0.000831506,-0.000831611,-0.000831715,-0.000831818,-0.00083192,-0.000832022,-0.000832123,-0.000832223,-0.000832322,-0.000837243,-0.000837341,-0.000837439,-0.000837537,-0.000837633,-0.000837729,-0.000837824,-0.000837918,-0.000838012,-0.000838105,-0.000838197,-0.000871722,-0.00087182,-0.000871917,-0.000872014,-0.00087211,-0.000872205,-0.000872299,-0.000872393,-0.000872486,-0.000872579,-0.00087267,-0.000872762,-0.00087806,-0.000878151,-0.000878241,-0.00087833,-0.000878419,-0.000878508,-0.000878595,-0.000878683,-0.000878769,-0.000878855,-0.00087894,-0.000879025,-0.000913342,-0.000913432,-0.000913521,-0.00091361,-0.000913698,-0.000913786,-0.000913873,-0.00091396,-0.000914045,-0.000914131,-0.000914215,-0.00089076,-0.000890839,-0.000890918,-0.000890996,-0.000891074,-0.000891151,-0.000891227,-0.000891303,-0.000891379,-0.000891454,-0.000891529,-0.000891603,-0.000926523,-0.000926602,-0.000926681,-0.000926759,-0.000926837,-0.000926914,-0.000926991,-0.000927067,-0.000927143,-0.000927218,-0.000927293,-0.000903592,-0.000903662,-0.000903731,-0.0009038,-0.000903869,-0.000903937,-0.000904005,-0.000904072,-0.000904139,-0.000904206,-0.000904272,-0.000904338,-0.000939853,-0.000939923,-0.000939993,-0.000940063,-0.000940132,-0.000940201,-0.00094027,-0.000940338,-0.000940405,-0.000940473,-0.000940539,-0.000916569,-0.000916632,-0.000916694,-0.000916755,-0.000916817,-0.000916878,-0.000916938,-0.000916999,-0.000917058,-0.000917118,-0.000917177,-0.000953276,-0.000953339,-0.000953403,-0.000953466,-0.000953529,-0.000953591,-0.000953653,-0.000953715,-0.000953776,-0.000953837,-0.000953897,-0.000953958,-0.000929697,-0.000929753,-0.000929809,-0.000929865,-0.00092992,-0.000929975,-0.00093003,-0.000930084,-0.000930138,-0.000930192,-0.000930246,-0.000936278,-0.000936332,-0.000936385,-0.000936438,-0.000936491,-0.000936544,-0.000936596,-0.000936648,-0.0009367,-0.000936752,-0.000936803,-0.000942898,-0.00094295,-0.000943001,-0.000943052,-0.000943103,-0.000943153,-0.000943203,-0.000943253,-0.000943303,-0.000943352,-0.000943402,-0.000918465,-0.00091851,-0.000918556,-0.000918601,-0.000918646,-0.00091869,-0.000918735,-0.000918779,-0.000918823,-0.000918866,-0.00091891,-0.000924914,-0.000924958,-0.000925001,-0.000925045,-0.000925088,-0.000925131,-0.000925174,-0.000925216,-0.000925259,-0.000925301,-0.000931356,-0.000931399,-0.000931441,-0.000931483,-0.000931525,-0.000931567,-0.000931608,-0.000931649,-0.00093169,-0.000931731,-0.000931772,-0.000937877,-0.000937918,-0.000937959,-0.000937999,-0.00093804,-0.00093808,-0.00093812,-0.00093816,-0.0009382,-0.000938239,-0.000938278,-0.000912433,-0.000912469,-0.000912505,-0.000912541,-0.000912577,-0.000912612,-0.000912647,-0.000912683,-0.000912718,-0.000912753,-0.000886508,-0.00088654,-0.000886572,-0.000886603,-0.000886635,-0.000886666,-0.000886697,-0.000886728,-0.000886759,-0.00088679,-0.00088682,-0.000925078,-0.000925113,-0.000925147,-0.00092518,-0.000925214,-0.000925248,-0.000925281,-0.000925315,-0.000925348,-0.000925381,-0.000898753,-0.000898784,-0.000898814,-0.000898844,-0.000898873,-0.000898903,-0.000898933,-0.000898962,-0.000898991,-0.000899021,-0.00089905,-0.000872004,-0.000872031,-0.000872057,-0.000872083,-0.000872109,-0.000872135,-0.00087216,-0.000872186,-0.000872212,-0.000872237,-0.000877921,-0.000877946,-0.000877972,-0.000877998,-0.000878023,-0.000878048,-0.000878074,-0.000878099,-0.000878124,-0.000878149,-0.00085048,-0.000850502,-0.000850524,-0.000850547,-0.000850569,-0.000850591,-0.000850612,-0.000850634,-0.000850656,-0.000850678,-0.000850699,-0.000856215,-0.000856237,-0.000856259,-0.000856281,-0.000856302,-0.000856324,-0.000856345,-0.000856367,-0.000856388,-0.000856409,-0.000861949,-0.000861971,-0.000861992,-0.000862014,-0.000862035,-0.000862056,-0.000862077,-0.000862098,-0.000862119,-0.00086214,-0.000799534,-0.000799549,-0.000799565,-0.00079958,-0.000799596,-0.000799611,-0.000799627,-0.000799642,-0.000799657,-0.000799673,-0.000839124,-0.000839142,-0.00083916,-0.000839178,-0.000839197,-0.000839215,-0.000839233,-0.00083925,-0.000839268,-0.000839286,-0.000775552,-0.000775565,-0.000775578,-0.00077559,-0.000775603,-0.000775615,-0.000775628,-0.00077564,-0.000775653,-0.000775665,-0.000780604,-0.000780617,-0.00078063,-0.000780642,-0.000780655,-0.000780667,-0.00078068,-0.000780692,-0.000780705,-0.000785658,-0.00078567,-0.000785683,-0.000785696,-0.000785708,-0.000785721,-0.000785733,-0.000785746,-0.000785758,-0.000785771},
		{0,6.70545e-18,5.31955e-12,4.09447e-10,3.51419e-09,1.30109e-08,3.20099e-08,6.2549e-08,1.05867e-07,1.62714e-07,2.3355e-07,3.18667e-07,4.18254e-07,5.32441e-07,6.61312e-07,8.0493e-07,9.63339e-07,1.13657e-06,1.32465e-06,1.52759e-06,1.74541e-06,1.97812e-06,2.22573e-06,2.48824e-06,2.76566e-06,3.058e-06,3.36525e-06,3.68742e-06,4.02451e-06,4.37653e-06,4.74347e-06,5.12533e-06,5.52213e-06,5.93385e-06,6.3605e-06,6.80208e-06,7.25859e-06,7.73004e-06,8.21641e-06,8.71771e-06,9.23395e-06,9.76512e-06,1.03112e-05,1.08722e-05,1.14482e-05,1.20391e-05,1.26449e-05,1.32657e-05,1.39014e-05,1.4552e-05,1.52175e-05,1.5898e-05,1.65935e-05,1.73038e-05,1.80291e-05,1.87693e-05,1.95244e-05,2.02945e-05,2.10795e-05,2.18794e-05,2.26943e-05,2.35241e-05,2.43688e-05,2.52285e-05,2.61031e-05,2.69926e-05,2.32779e-05,2.33711e-05,2.34605e-05,2.35465e-05,2.36291e-05,2.37086e-05,2.37851e-05,2.38588e-05,2.39297e-05,2.39981e-05,2.4064e-05,2.41275e-05,2.41889e-05,4.64564e-05,4.66191e-05,4.67763e-05,4.69283e-05,4.70752e-05,4.72173e-05,4.73548e-05,4.74879e-05,4.76168e-05,4.77416e-05,4.78626e-05,4.79798e-05,4.80935e-05,4.82906e-05,4.83978e-05,4.85019e-05,4.8603e-05,4.87011e-05,4.87963e-05,4.88889e-05,4.89789e-05,4.90663e-05,4.91514e-05,4.92341e-05,4.93145e-05,4.93928e-05,7.28979e-05,7.30351e-05,7.31688e-05,7.32989e-05,7.34257e-05,7.35493e-05,7.36697e-05,7.37872e-05,7.39016e-05,7.40133e-05,7.41222e-05,7.42285e-05,7.43322e-05,7.46054e-05,7.47046e-05,7.48015e-05,7.48961e-05,7.49884e-05,7.50787e-05,7.51669e-05,7.52531e-05,7.53374e-05,7.54198e-05,7.55003e-05,7.55791e-05,7.56562e-05,0.000100041,0.000100157,0.00010027,0.000100381,0.00010049,0.000100596,0.0001007,0.000100802,0.000100902,0.000101,0.000101096,0.00010119,0.000101282,0.000126093,0.000126216,0.000126338,0.000126457,0.000126573,0.000126688,0.0001268,0.00012691,0.000127018,0.000127124,0.000127228,0.00012733,0.000127431,0.000152642,0.000152769,0.000152893,0.000153015,0.000153135,0.000153253,0.000153369,0.000153483,0.000153595,0.000153705,0.000153813,0.000153919,0.000154024,0.00015463,0.000154732,0.000154831,0.000154929,0.000155026,0.000155121,0.000155214,0.000155306,0.000155396,0.000155485,0.000155572,0.000155658,0.000181497,0.000181604,0.000181709,0.000181812,0.000181913,0.000182013,0.000182112,0.000182209,0.000182304,0.000182398,0.000182491,0.000182582,0.000182672,0.000208884,0.000208992,0.000209098,0.000209203,0.000209305,0.000209407,0.000209507,0.000209606,0.000209703,0.000209798,0.000209893,0.000209986,0.000210078,0.000210989,0.000211078,0.000211167,0.000211254,0.000211341,0.000211426,0.000211509,0.000211592,0.000211674,0.000211754,0.000211834,0.000211912,0.000264609,0.000264718,0.000264825,0.000264931,0.000265035,0.000265138,0.00026524,0.000265341,0.00026544,0.000265538,0.000265635,0.00026573,0.000265825,0.000267059,0.000267152,0.000267243,0.000267334,0.000267423,0.000267511,0.000267598,0.000267684,0.000267769,0.000267853,0.000267936,0.000268018,0.000268099,0.000269377,0.000269456,0.000269535,0.000269613,0.00026969,0.000269766,0.000269841,0.000269915,0.000269989,0.000270061,0.000270133,0.000270204,0.000324513,0.000324607,0.000324699,0.000324791,0.000324881,0.000324971,0.000325059,0.000325147,0.000325233,0.000325319,0.000325403,0.000325487,0.000327134,0.000327217,0.000327299,0.00032738,0.00032746,0.000327539,0.000327617,0.000327694,0.000327771,0.000327847,0.000327922,0.000327996,0.00032807,0.000356661,0.000356744,0.000356826,0.000356907,0.000356987,0.000357066,0.000357145,0.000357222,0.000357299,0.000357376,0.000357451,0.000357526,0.000359424,0.000359498,0.000359571,0.000359643,0.000359715,0.000359786,0.000359856,0.000359926,0.000359995,0.000360063,0.000360131,0.000360198,0.000389434,0.000389509,0.000389583,0.000389657,0.00038973,0.000389802,0.000389873,0.000389944,0.000390015,0.000390084,0.000390153,0.000390221,0.000390289,0.000392451,0.000392518,0.000392585,0.00039265,0.000392716,0.00039278,0.000392844,0.000392908,0.000392971,0.000393033,0.000393095,0.000393156,0.000423062,0.00042313,0.000423198,0.000423265,0.000423331,0.000423397,0.000423463,0.000423527,0.000423592,0.000423655,0.000423719,0.000423781,0.000426217,0.00042628,0.000426341,0.000426402,0.000426463,0.000426523,0.000426583,0.000426642,0.0004267,0.000426759,0.000426816,0.000426873,0.000457455,0.000457518,0.000457581,0.000457644,0.000457706,0.000457767,0.000457828,0.000457888,0.000457948,0.000458008,0.000458067,0.000458125,0.000460843,0.000460901,0.000460959,0.000461017,0.000461073,0.00046113,0.000461186,0.000461241,0.000461297,0.000461351,0.000461406,0.000464178,0.000464232,0.000464286,0.000464339,0.000464392,0.000464444,0.000464496,0.000464548,0.000464599,0.00046465,0.000464701,0.000464751,0.000496276,0.000496332,0.000496387,0.000496442,0.000496496,0.00049655,0.000496603,0.000496656,0.000496709,0.000496762,0.000496814,0.000496865,0.000499925,0.000499976,0.000500027,0.000500078,0.000500128,0.000500178,0.000500228,0.000500277,0.000500326,0.000500375,0.000500423,0.000503534,0.000503582,0.00050363,0.000503678,0.000503725,0.000503772,0.000503818,0.000503864,0.00050391,0.000503956,0.000504001,0.000504046,0.000507207,0.000507252,0.000507296,0.000507341,0.000507384,0.000507428,0.000507471,0.000507514,0.000507557,0.0005076,0.000507642,0.000540388,0.000540435,0.000540481,0.000540527,0.000540573,0.000540619,0.000540664,0.000540709,0.000540754,0.000540799,0.000540843,0.000540887,0.000514581,0.000514621,0.00051466,0.000514699,0.000514737,0.000514776,0.000514814,0.000514852,0.00051489,0.000514927,0.000514964,0.000548229,0.000548271,0.000548312,0.000548353,0.000548394,0.000548434,0.000548474,0.000548514,0.000548554,0.000548594,0.000548633,0.000552171,0.00055221,0.000552249,0.000552288,0.000552326,0.000552365,0.000552403,0.000552441,0.000552478,0.000552516,0.000552553,0.00055259,0.00055617,0.000556207,0.000556244,0.00055628,0.000556317,0.000556353,0.000556389,0.000556424,0.00055646,0.000556495,0.00055653,0.000529527,0.000529558,0.00052959,0.000529621,0.000529651,0.000529682,0.000529712,0.000529743,0.000529773,0.000529803,0.000529832,0.000564127,0.000564161,0.000564194,0.000564227,0.00056426,0.000564293,0.000564326,0.000564358,0.00056439,0.000564422,0.000564454,0.000568147,0.000568179,0.000568211,0.000568242,0.000568274,0.000568305,0.000568336,0.000568367,0.000568398,0.000568429,0.000568459,0.000572186,0.000572216,0.000572247,0.000572277,0.000572307,0.000572337,0.000572367,0.000572396,0.000572426,0.000572455,0.000576213,0.000576242,0.000576272,0.000576301,0.00057633,0.000576358,0.000576387,0.000576415,0.000576444,0.000576472,0.0005765,0.000548553,0.000548577,0.000548602,0.000548627,0.000548651,0.000548676,0.0005487,0.000548724,0.000548748,0.000548772,0.000548795,0.000584361,0.000584388,0.000584415,0.000584442,0.000584469,0.000584495,0.000584522,0.000584548,0.000584574,0.0005846,0.000588444,0.00058847,0.000588496,0.000588522,0.000588547,0.000588573,0.000588598,0.000588624,0.000588649,0.000588674,0.000588699,0.00056015,0.000560172,0.000560194,0.000560216,0.000560238,0.000560259,0.000560281,0.000560302,0.000560324,0.000560345,0.000596665,0.000596689,0.000596713,0.000596737,0.000596761,0.000596785,0.000596809,0.000596833,0.000596856,0.00059688,0.000596903,0.000567941,0.000567961,0.000567982,0.000568002,0.000568022,0.000568043,0.000568063,0.000568083,0.000568103,0.000568123,0.000604944,0.000604967,0.00060499,0.000605012,0.000605035,0.000605057,0.000605079,0.000605102,0.000605124,0.000605146,0.000575763,0.000575782,0.000575802,0.000575821,0.00057584,0.000575859,0.000575878,0.000575897,0.000575916,0.000575935,0.000575953,0.000579703,0.000579722,0.000579741,0.000579759,0.000579778,0.000579796,0.000579815,0.000579833,0.000579851,0.00057987,0.000583635,0.000583654,0.000583672,0.00058369,0.000583708,0.000583726,0.000583744,0.000583762,0.00058378,0.000583797,0.000587578,0.000587596,0.000587613,0.000587631,0.000587649,0.000587666,0.000587684,0.000587701,0.000587718,0.000587736,0.000591529,0.000591547,0.000591564,0.000591582,0.000591599,0.000591616,0.000591633,0.00059165,0.000591667,0.000591684,0.000629993,0.000630012,0.000630032,0.000630051,0.000630071,0.00063009,0.000630109,0.000630128,0.000630147,0.000630166,0.000599472,0.000599488,0.000599505,0.000599521,0.000599538,0.000599554,0.00059957,0.000599587,0.000599603,0.000603431,0.000603447,0.000603463,0.000603479,0.000603495,0.000603511,0.000603527,0.000603543,0.000603559,0.000603575}};
//
	for(ichannel=0;ichannel<3;ichannel++){
		delta[ichannel][0]=ddeltadq[ichannel][0]=0.0;
		if(ichannel==0) ddeltadq[0][0]=data_delta[0][1]-data_delta[0][0];
		for(iq=1;iq<nqmax;iq++){
			q=qarray[iq];
			iqdata=int(floor(q)/delqdata);
			if(iqdata<nqdata){
				w1=(delqdata*double(iqdata+1)-q)/delqdata;
				w2=1.0-w1;
				delta[ichannel][iq]=w1*data_delta[ichannel][iqdata]
					+w2*data_delta[ichannel][iqdata];
				ddeltadq[ichannel][iq]=w1*data_ddeltadq[ichannel][iqdata]
					+w2*data_ddeltadq[ichannel][iqdata];
			}
			else{
				delta[ichannel][iq]=ddeltadq[ichannel][iq]=0.0;
			}
		}
	}
}

#endif
